#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.control.mov_caixa import MovCaixa
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.models.cad_operadores_data import CaixaCadOperadores
from sigpdv.models.db_utils import exec_sql
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.control.utils import atualizar_saldo_inicial_supervisor
from sigpdv.ui.messagebox import MSG_INFO, NO_ROLE, MSG_QUESTION, message_dlg, YES_ROLE


class AberturaFechamentoTurno(object):
    def __init__(self, parent):
        self.parent = parent

    @Log
    def abrir_turno(self, loja, caixa, cod_operador):
        """
        Realiza a abertura do turno para o operador informado.
        """
        if self.parent.verifica_z_emitido():
            message_dlg(self.parent, MSG_INFO, 'Redução Z de hoje já foi emitida!\n'
                                        'Não é possivel abrir turno, deve-se esperar o próximo'
                                        ' dia fiscal!')
            return False

        cad_operador = CaixaCadOperadores.get_by(supope_codigo=cod_operador)
        msg = 'Ola %s.\n\nDeseja abrir um novo turno?' % str(cad_operador.name)
        if message_dlg(self.parent, MSG_QUESTION, msg) == NO_ROLE:
            return False

        data_hora_abertura = self.parent.obj_fiscal_printer.get_data_ecf()
        try:
            if not data_hora_abertura:
                if message_dlg(self.parent, MSG_QUESTION, 'Nao foi possivel recuperar data da ECF'
                                                   '\nDeseja tentar novamente?') == YES_ROLE:
                    return self.abrir_turno(loja, caixa, cod_operador)
                else:
                    message_dlg(self.parent, MSG_INFO, 'Nao foi possivel recuperar data da ECF'
                                                '\nVerifique a conexao com a ECF')
                    return False
            self.parent.mov_caixa = MovCaixa().create(
                dict(supcax_loja=loja,
                     supcax_caixa=caixa,
                     supcax_data_hora_abertura=data_hora_abertura,
                     supcax_operador_id=cad_operador.id,
                     supcax_fechado='A',
                     supcax_enviado='N'))

            if not self.parent.mov_caixa:
                return False

            self.parent.ui.lblNumCaixa.setText(u'%03d <b>Aberto' % caixa)
            self.parent.ui.mainMenu.item(1).setText(u'Fechar Turno')  # imenu_turno
        except Exception as err:
            Log.error(str(err))

        message_dlg(self.parent, MSG_INFO,
                    'Turno aberto:\n\nOperador: %s\nCod. Cx.: %d\nLoja: %.4d\nData/Hora: %s'
                    % (str(cad_operador.name), caixa, loja, data_hora_abertura))

        return True

    @Log
    def fechar_turno(self, mov_caixa, loja, caixa, operador=(0, '')):
        """
        Realiza o fechamento de turno e imprime um relatório gerencial.

        :param mov_caixa_id: id do turno
        :param loja: loja_id
        :param caixa: caixa_id
        :param operador: tupla com o id e o nome do operador
        :return: boolean
        """
        status = self.parent.obj_fiscal_printer.send_command("get_status()")
        if status.open or status.pagamento_existe:
            self.parent.obj_cupom_fiscal.cancelar_cupom(tipo_cancelamento='N')

        texto = 'Relatório: Fechamento de Turno'
        if self.parent.teclado:
            self.parent.teclado.envia_display('Imprimindo %s...' % texto)

        self.parent.show_info_proccess(True, texto)
        try:
            data_hora_fechamento = self.parent.obj_fiscal_printer.get_data_ecf()
            text = self.get_valores_fechamento_turno(mov_caixa, loja, caixa, operador,
                                                     data_hora_fechamento.strftime('%H:%M:%S'))
            if not text:
                raise Exception('Erro ao fechar o turno. Verifique se o turno já foi fechado.')

            GerencialReport(report_id=config['rg_relatorio_gerencial'],
                            text=text,
                            loja_id=loja,
                            caixa_id=caixa,
                            operador_id=operador[0],
                            tipo='F',
                            parent=self.parent)

            MovCaixa().write(mov_caixa,
                             {'supcax_data_hora_fechamento': data_hora_fechamento,
                              'supcax_fechado': 'F', 'supcax_enviado': 'N'})

            self.parent.show_info_proccess(False)

            self.parent.ui.mainMenu.item(1).setText(u'Abrir Turno')  # imenu_turno
            data_fechamento = self.parent.mov_caixa.supcax_data_hora_abertura.strftime('%d/%m/%Y')
            message_dlg(self.parent, MSG_INFO, "O turno do dia: %s foi fechado!" % str(data_fechamento))
            self.parent.mov_caixa = False

        except Exception as err:
            self.parent.show_info_proccess(False)
            Log.error(str(err))
            return False

        return True

    def get_saldo_turno(self, mov_caixa, loja, caixa, operador=(0, '')):
        """
        Retorna o saldo em caixa para o turno informado.

        :param mov_caixa: turno
        :param loja: loja_id
        :param caixa: caixa_id
        :param operador: tupla com o id e o nome do operador
        :return: float com o valor do saldo em caixa
        """
        if not mov_caixa:
            return 0

        saldo = self.get_valores_fechamento_turno(mov_caixa, loja, caixa, operador, get_saldo=True)
        return saldo

    @Log
    def get_valores_fechamento_turno(self, mov_caixa, loja, caixa, operador=(0, ''),
                                     hora_fechamento='', get_saldo=False):
        """
        Se get_saldo = False, retorna o texto à ser impresso no relatório gerencial
        de fechamento de turno.
        Se get_saldo = True, retorna o valor total do saldo em caixa para o turno informado.
        """

        data_mov = MovCaixa().get_data_hora_abertura(loja, caixa, mov_caixa.id)

        if not data_mov:
            return False

        totais_dinheiro = 0
        qtds_dinheiro = 0

        totais_cheque = 0
        qtds_cheque = 0

        totais_cartao = 0
        qtds_cartao = 0

        totais_ticket = 0
        qtds_ticket = 0

        totais_outros = 0
        qtds_outros = 0

        texto = "                              \n"

        for x in range(2):  # 0=Venda 1=Cancelamento
            tipo = 'V' if x == 0 else 'C'

            sql = """
                SELECT Sum(cmf.supfec_valor - cmf.supfec_troco) AS totvalor,
                       Count(cmf.supfec_tipo_finalizadora)      AS qtdfin,
                       cmf.supfec_tipo_finalizadora             AS tipo_finalizadora,
                       cmc.supcap_tipo                          AS tipo
                FROM caixa_mov_fechamentos AS cmf
                INNER JOIN caixa_mov_capa  AS cmc ON cmc.id = cmf.supfec_capa_id
                WHERE cmc.supcap_data_fim > '{data_mov}'
                  AND cmc.supcap_tipo = '{tipo}'
                  AND cmc.supcap_loja = {loja}
                  AND cmc.supcap_caixa = {caixa}
                GROUP BY cmf.supfec_tipo_finalizadora, cmc.supcap_tipo
            """.format(tipo=tipo,
                       data_mov=data_mov,
                       loja=loja,
                       caixa=caixa)

            for dados in exec_sql(sql, 'C', ret=True):
                sinal = 1 if tipo == 'V' else -1 # venda ou cancelamento
                if dados.get('tipo_finalizadora') == 'D':
                    totais_dinheiro += (dados.get('totvalor') or 0) * sinal
                    qtds_dinheiro += (dados.get('qtdfin') or 0) * sinal

                elif dados.get('tipo_finalizadora') == 'H':
                    totais_cheque += (dados.get('totvalor') or 0) * sinal
                    qtds_cheque += (dados.get('qtdfin') or 0) * sinal

                elif dados.get('tipo_finalizadora') == 'C':
                    totais_cartao += (dados.get('totvalor') or 0) * sinal
                    qtds_cartao += (dados.get('qtdfin') or 0) * sinal

                elif dados.get('tipo_finalizadora') == 'T':
                    totais_ticket += (dados.get('totvalor') or 0) * sinal
                    qtds_ticket += (dados.get('qtdfin') or 0) * sinal

                elif dados.get('tipo_finalizadora') == 'O':
                    totais_outros += (dados.get('totvalor') or 0) * sinal
                    qtds_outros += (dados.get('qtdfin') or 0) * sinal

        ############################################################################################

        texto += "Data Mov.  : %s\n" % data_mov.strftime('%d/%m/%Y')
        texto += "Operador   : %03d-%s\n" % operador
        texto += "Abertura   : %s\n" % data_mov.strftime('%H:%M:%S')
        texto += "Fechamento : %s\n" % str(hora_fechamento)
        texto += "\n"
        texto += "* %s\n" % 'Vendas Liquidas'
        texto += "------------ --- ------------\n"
        texto += "Finalizadora Qtd        Valor\n"
        texto += "------------ --- ------------\n"

        texto += "Dinheiro     %03d %12.2f\n" % (qtds_dinheiro, totais_dinheiro)
        texto += "Cheque       %03d %12.2f\n" % (qtds_cheque, totais_cheque)
        texto += "Cartao       %03d %12.2f\n" % (qtds_cartao, totais_cartao)
        texto += "Ticket       %03d %12.2f\n" % (qtds_ticket, totais_ticket)
        texto += "Outros       %03d %12.2f\n" % (qtds_outros, totais_outros)

        # ----------------------------------- Total das Vendas ----------------------------------- #
        soma = sum([totais_dinheiro,
                    totais_cheque,
                    totais_cartao,
                    totais_ticket,
                    totais_outros])

        texto += "-----------------------------\n"
        texto += "Total de Vendas: %12.2f\n" % soma
        texto += " \n"

        ############################################################################################

        # ----------------------------------- Sangria -------------------------------------------- #
        sql = """

            SELECT SUM(cmf.supfec_valor)               AS totvalor,
                   COUNT(cmf.supfec_tipo_finalizadora) AS qtdfin,
                   cmf.supfec_tipo_finalizadora        AS tipo_finalizadora
            FROM caixa_mov_fechamentos AS cmf
            INNER JOIN caixa_mov_capa  AS cmc ON cmc.id = cmf.supfec_capa_id
            WHERE cmc.supcap_data_fim > '{data_mov}'
              AND cmc.supcap_tipo = 'A'
              AND cmc.supcap_loja={loja}
              AND cmc.supcap_caixa={caixa}
            GROUP BY cmf.supfec_tipo_finalizadora
        """.format(data_mov=data_mov,
                   loja=loja,
                   caixa=caixa)

        total_sd = 0
        total_sh = 0
        total_sc = 0
        total_st = 0
        total_so = 0
        qtda_sd = 0
        qtda_sh = 0
        qtda_sc = 0
        qtda_st = 0
        qtda_so = 0
        texto += "* SANGRIAS\n"
        texto += "------------ --- ------------\n"
        texto += "Finalizadora Qtd        Valor\n"
        texto += "------------ --- ------------\n"
        for dados in exec_sql(sql, 'C', ret=True):
            if dados.get('tipo_finalizadora') == 'D':
                total_sd = dados.get('totvalor') or 0
                qtda_sd = dados.get('qtdfin') or 0

            elif dados.get('tipo_finalizadora') == 'H':
                total_sh = dados.get('totvalor') or 0
                qtda_sh = dados.get('qtdfin') or 0

            elif dados.get('tipo_finalizadora') == 'C':
                total_sc = dados.get('totvalor') or 0
                qtda_sc = dados.get('qtdfin') or 0

            elif dados.get('tipo_finalizadora') == 'T':
                total_st = dados.get('totvalor') or 0
                qtda_st = dados.get('qtdfin') or 0

            elif dados.get('tipo_finalizadora') == 'O':
                total_so = dados.get('totvalor') or 0
                qtda_so = dados.get('qtdfin') or 0

        texto += "Dinheiro     %03d %12.2f\n" % (qtda_sd, total_sd)
        texto += "Cheque       %03d %12.2f\n" % (qtda_sh, total_sh)
        texto += "Cartao       %03d %12.2f\n" % (qtda_sc, total_sc)
        texto += "Ticket       %03d %12.2f\n" % (qtda_st, total_st)
        texto += "Outros       %03d %12.2f\n" % (qtda_so, total_so)

        # ---------------------------------- Total das Sangrias ---------------------------------- #
        texto += "-----------------------------\n"
        texto += "Total Sangrias.: %12.2f\n" % sum([total_sd, total_sh, total_sc,
                                                    total_st, total_so])
        texto += " \n"

        # -------------------------------------- Suprimento -------------------------------------- #
        sql = """

            SELECT SUM(supcap_total_cupom) AS totalcupom,
                   COUNT(supcap_nro_cupom) AS qtdsup
            FROM  caixa_mov_capa
            WHERE supcap_data_fim > '%s'
                  AND supcap_tipo='U'
                  AND supcap_loja=%d
                  AND supcap_caixa=%d

        """ % (data_mov, loja, caixa)

        ret_sql_su = exec_sql(sql, 'C', ret=True)

        total_su = ret_sql_su[0].get('totalcupom') if ret_sql_su[0].get('totalcupom') else 0
        qtda_su = ret_sql_su[0].get('qtdsup')

        texto += "* SUPRIMENTO (%03d) %10.2f\n" % (qtda_su, total_su)

        # -------------------------------------- Contra-Vale ------------------------------------- #
        sql = """

              SELECT SUM(supcap_total_cupom) AS totvalor,
                     COUNT(supcap_loja)      AS qtdfin
              FROM   caixa_mov_capa
              WHERE  supcap_tipo = 'O'
                     AND supcap_loja = %d
                     AND supcap_caixa = %d
                     AND supcap_data_fim > '%s'

        """ % (loja, caixa, data_mov)

        ret_sql_cv = exec_sql(sql, 'C', ret=True)

        total_cv = ret_sql_cv[0].get('totvalor') if ret_sql_cv[0].get('totvalor') else 0
        qtda_cv = ret_sql_cv[0].get('qtdfin')

        texto += "* CONTRA-VALE(%03d) %10.2f\n" % (qtda_cv, total_cv)
        dados_ficha_doacao = self.get_dados_ficha_doacao(data_mov, loja, caixa)
        if dados_ficha_doacao:
            if dados_ficha_doacao.get(config['tot_ficha']):
                texto += "* Ficha " + ("%10.2f\n" % dados_ficha_doacao.get(
                                                                     config['tot_ficha'])).rjust(22)
            if dados_ficha_doacao.get(config['tot_devolucao_ficha']):
                texto += "* Devolucao Ficha " + ("%10.2f\n" % dados_ficha_doacao.get(
                                                           config['tot_devolucao_ficha'])).rjust(12)
            if dados_ficha_doacao.get(config['totalizador_doacao']):
                texto += "* Doacao " +  ("%10.2f\n" % dados_ficha_doacao.get(
                                                            config['totalizador_doacao'])).rjust(21)

        texto += " \n"

        # ------------------------------------ Saldo em Caixa ------------------------------------ #
        tot_d = totais_dinheiro + total_su - total_sd
        tot_h = totais_cheque - total_sh
        tot_c = totais_cartao - total_sc
        tot_t = totais_ticket - total_st
        tot_o = totais_outros - total_so

        tot_geral = sum([tot_d, tot_h, tot_c, tot_t, tot_o, total_cv])

        texto += "******* SALDO EM CAIXA ******\n"
        texto += " \n"

        if tot_geral:
            if tot_d:
                texto += "Dinheiro...: %16.2f\n" % tot_d
            if tot_h:
                texto += "Cheque.....: %16.2f\n" % tot_h
            if tot_c:
                texto += "Cartao.....: %16.2f\n" % tot_c
            if tot_t:
                texto += "Ticket.....: %16.2f\n" % tot_t
            if tot_o:
                texto += "Outros.....: %16.2f\n" % tot_o

            texto += " \n"
            texto += "TOTAL GERAL: %16.2f\n" % tot_geral
        else:
            texto += "******* SALDO EM CAIXA ******\n"
            texto += " \n"
            texto += "::: NAO HA SALDO EM CAIXA :::\n"

        texto += " \n"
        texto += "*****************************"
        texto += " \n"

        if get_saldo:
            res = tot_geral
        else:
            res = str(texto)

        return res

    def get_dados_ficha_doacao(self, data_mov, loja, caixa):
        sql = """
                SELECT SUM(cmc.supcap_total_cupom)   AS total, 
                       cmc.supcap_cod_tot_nao_fiscal AS totalizador
                FROM caixa_mov_capa  AS cmc
                WHERE cmc.supcap_data_fim > '{data_mov}'
                  AND cmc.supcap_tipo = '{tipo}'
                  AND cmc.supcap_loja = {loja}
                  AND cmc.supcap_caixa = {caixa}
                  AND cmc.supcap_cod_tot_nao_fiscal IN ({tot})
                GROUP BY cmc.supcap_cod_tot_nao_fiscal
        """.format(tipo='W',
                   data_mov=data_mov,
                   loja=loja,
                   caixa=caixa,
                   tot= '%s, %s, %s' % (config['tot_ficha'] or 0,
                                        config['tot_devolucao_ficha'] or 0,
                                        config['totalizador_doacao'] or 0))
        res = exec_sql(sql, 'C', ret=True)
        if not res:
            return False

        vals = {}
        map(lambda x: vals.update({x.get('totalizador'): x.get('total')}), res)
        return vals


