#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime, timedelta
from decimal import Decimal, ROUND_DOWN
from PySide import QtCore, QtGui
from stoqdrivers.exceptions import ItemAdditionError, CancelItemError, CouponNotOpenError, \
    TimeoutError, OutofPaperError, CouponOpenError
from sigpdv import root_path
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.control.cad_tributacao import CadTributacao
from sigpdv.control.mov_corpos import MovCorpos
from sigpdv.control.mov_fechamentos import MovFechamentos
from sigpdv.control.mov_vencimentos import MovVencimentos
from sigpdv.control.cad_kits import CadKits
from sigpdv.control.mov_kits_vendidos import MovKitsVendidos
from sigpdv.control.mov_meios_pagamentos import MovMeiosPagamentos
from sigpdv.control.cad_clientes import CadClientes
from sigpdv.control.mov_autorizacoes import MovAutorizacoes
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.control.utils import ler_arq_socio_torcedor
from sigpdv.control.utils import remove_accents, checar_conexao
from sigpdv.control.sorteio import CadSorteio
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.cad_finalizadora_data import CaixaCadFinalizadora
from sigpdv.models.cad_finzalizadora_na_ecf_data import CaixaCadFinalizadoraNaEcf
from sigpdv.models.mov_fechamentos_data import CaixaMovFechamentos
from sigpdv.models.cad_mensagens_clientes_data import CaixaCadMensagensClientes
from sigpdv.models.cfg_mensagens_data import CaixaCfgMensagens
from sigpdv.models.supervisor import SupervisorPveFechamento
from sigpdv.models.db_utils import b_sess, exec_sql
from sigpdv.dav_pve.dav_gestao import DavGestao
from sigpdv.tools.log import traced as Log
from sigpdv.tools.config import config
from sigpdv.tools.cardinals import NumeroPorExtenso
from sigpdv.ui.dlpPwd import password_dlg
from sigpdv.ui.messagebox import YES_ROLE, NO_ROLE, MSG_ERROR, MSG_QUESTION, MSG_INFO, \
    MSG_NONE, message_dlg
import time
import calendar
import re
import os

TWOPLACES = Decimal(10) ** -2
TREPLACES = Decimal(10) ** -3

RESTRICOES = {1: u'Restrição de venda por item!',
              2: u'Restrição de venda por quantidade!'}

retricao_item, restricao_qtd = 1,2

# STATUS DO CUPOM
coupon_closed, coupon_opened, coupon_add_discount, coupon_add_surchage, coupon_add_payment, \
coupon_finish, mesa_closed, mesa_opened, mesa_transf, itens_mesa_transf = range(10)

# STATUS DO ITEM
item_none, item_add_quantity, item_add_discount, item_add_surchage, item_cancel, item_balanca = range(6)

# Metodos de Pagamento
pm_dinheiro, pm_cheque, pm_cartao, pm_ticket, pm_outros, pm_end = range(6)

# STATUS DA TELA
in_login, in_menu, in_menu_fiscal, in_coupon, in_ficha, in_close_turn, in_ss, in_search_client, \
in_add_payment, in_dav, in_pve, in_importacao, in_item_menu_fiscal, in_end_sale, \
in_restaurante, in_update, in_troca_ficha = range(17)

stk_page_menu, stk_page_login, stk_page_coupom, stk_page_ficha, stk_page_payments, stk_page_ss, \
    stk_search_client, stk_dav_pre, stk_tef, stk_fiscal, stk_importacao, stk_restaurant, \
    stk_consulta_ch, stk_update = range(14)


class CupomFiscal(object):
    def __init__(self, parent):
        self.parent = parent
        self.totalizadores_ant_cupom = [] # totalizadores antes do cupom ser vendido

    def _cancel(self):
        '''
        Cancela o ultimo cupom aberto
        '''
        self.parent.send_command("cancel()")

    @Log
    def get_totalizadores(self, open_coupon=False):
        '''
        Coloca os totalizadores na variavel 'totalizadores_ant_cupom', ou seja, os totalizadores antes da
        venda do cupom para depois comparar com os totalizadores depois que o cupom for feito.
        :param open_coupon: Deve ser True se a chamada da funcao vier de open_coupon()
        :return: lista de tuplas, onde o primeiro elemento da tupla eh o nome da tributacao, o segundo eh o
                 valor total da tributacao do dia e se houver terceiro elemento
                 indica se eh ICMS ou ISSQN
        '''
        # pega os totalizadores ao abrir o cupom
        if open_coupon:
            self.totalizadores_ant_cupom = self.parent.send_command('get_totalizadores()')
            f = open('totalizadores.txt', 'w')
            for tot in self.totalizadores_ant_cupom:
                str_tot = str(tot[0])
                linha = str_tot + '-' + str(tot[1]) + '-' + str(tot[2]) + ';'
                f.write(linha)
                if str_tot not in  self.parent.list_totalizadores:
                    self.parent.list_totalizadores.append(str_tot)

            f.close()
            return True

        try:  # recupera os totalizadores de antes de iniciar venda do cupom que esta sendo recuperado
            self.totalizadores_ant_cupom = []
            f = open('totalizadores.txt', 'r')
            linha = f.read()
            for i in linha.split(';'):
                if not i:
                    continue

                tot = i.split('-')
                trib = (tot[0], Decimal(tot[1]), tot[2])
                self.totalizadores_ant_cupom.append(trib)
                if tot[0] not in  self.parent.list_totalizadores:
                    self.parent.list_totalizadores.append(tot[0])

        except Exception as e:
            Log.error('\nNão conseguiu recuperar os totalizadores do cupom em aberto! Erro: %s' % str(e))
            return False

        return True

    @Log
    def check_coupon_is_opened(self):
        """
        Verifica se existe algum cupom fiscal aberto na ECF. Se houver, pergunta ao usuário se
        deseja tentar recuperar ou anular/cancelar o cupom fiscal.
        :return: Boolean
        """
        if self.parent._state_coupon == coupon_opened:
            return True

        if self.parent._printer.brand in ['elgin', 'daruma']:
            self.parent.send_command('get_coo()')

        status = self.parent.send_command("get_status()")
        if status.open or status.pagamento_existe:
            # recuperra nsu do socio torcedor se houver
            try:
                self.parent._tef_nsu_socio_torcedor = ler_arq_socio_torcedor()
                if self.parent._tef_nsu_socio_torcedor:
                    self.parent.sig_tef.usando_sitef = True
                    self.parent.sig_tef.modalidade.value = 64
            except:
                pass

            msg = 'Existe um Cupom em aberto. Deseja tentar recuperar o Cupom?'
            if self.parent.linhas_display == 2:
                msg = 'Cupom em aberto. Tentar recuperar?'

            if message_dlg(self.parent, MSG_QUESTION, msg) == NO_ROLE:

                coo = self.cancelar_cupom(tipo_cancelamento='N')
                message_dlg(self.parent, MSG_INFO, 'Cupom %06d, cancelado.' % coo)
                self.parent._clear_globals(coupon_closed)
                self.parent.clear_widget_table(self.parent.ui.tableItens)
                return False

            coo = self.parent.send_command("get_coo()")
            data_cupom = datetime.date(self.parent.obj_fiscal_printer.get_data_ecf())
            mov_capa = self.parent.obj_mov_capa.search({'supcap_loja': config['loja'],
                                         'supcap_caixa': config['caixa'],
                                         'supcap_nro_cupom': coo,
                                         'supcap_data_cupom': data_cupom})

            if not mov_capa:
                self._cancel()
                self.parent.emitindo_dav = False
                return False

            self.parent.mov_capa = mov_capa
            self.parent._coupon = mov_capa

            acres_desc = mov_capa.supcap_alteracao
            valor_desc_acres = mov_capa.supcap_valor_desc_acres

            if acres_desc == 'D':
                self.parent._coupon_amount = self.parent._coupon_amount - valor_desc_acres
                self.parent._indicador_desc_acre = 2

            elif acres_desc == 'A':
                self.parent._coupon_amount = self.parent._coupon_amount + valor_desc_acres
                self.parent._indicador_desc_acre = 3

            total_cupom = mov_capa.supcap_total_cupom

            # Verifica se o Total do cupom é igual ao total da ecf
            if total_cupom != self.parent.send_command("get_coupon_subtotal()"):
                coo = self.cancelar_cupom(tipo_cancelamento='N')
                if coo:
                    message_dlg(self.parent, MSG_INFO, 'Cupom Cancelado!\n\n'
                                                'Não foi possível recuperar o cupom %06d.' % coo)
                self.parent._clear_globals(coupon_closed)
                self.parent._set_controls_default_values()
                return False

            if CaixaMovFechamentos.get_by(supfec_capa_id=mov_capa.id):
                coo = self.cancelar_cupom(tipo_cancelamento='N')
                message_dlg(self.parent, MSG_INFO, 'Cupom Cancelado!\n\n'
                                            'Não foi possível recuperar o cupom %06d.' % coo)
                self.parent._clear_globals(coupon_closed)
                self.parent._set_controls_default_values()
                return False

            # Verifica Cliente
            if mov_capa.supcap_cliente:
                self.parent._search_cliente(mov_capa.supcap_cliente)

            mov_corpos = MovCorpos(self.parent).search({'supcor_capa_id': mov_capa.id},
                                                  order_by='supcor_sequencia')

            if not mov_corpos and total_cupom:
                coo = self.cancelar_cupom(tipo_cancelamento='N')
                message_dlg(self.parent, MSG_INFO, 'Cupom Cancelado!\n\n'
                                            'Não foi possível recuperar o cupom %06d.' % coo)
                self.parent._clear_globals(coupon_closed)
                self.parent._set_controls_default_values()
                return False

            if not isinstance(mov_corpos, list):
                mov_corpos = [mov_corpos]

            self.parent.mov_corpos = mov_corpos
            sum_itens_cupom = sum(map(
                lambda c: c.supcor_preco_total + c.supcor_acrescimo_item - c.supcor_desconto_item
                          if c.supcor_venda_cancelamento == 'V' else 0,
                mov_corpos))

            if total_cupom != sum_itens_cupom:
                coo = self.cancelar_cupom(tipo_cancelamento='N')
                message_dlg(self.parent, MSG_INFO, 'Cupom Cancelado!\n\n'
                                            'Não foi possível recuperar o cupom %06d.' % coo)
                self.parent._clear_globals(coupon_closed)
                self.parent._set_controls_default_values()
                return False

            for corpo in mov_corpos:
                sequencia = corpo.supcor_sequencia - 1
                self.parent.last_item = int(corpo.supcor_sequencia)
                cod_ficha = False
                if config['usar_ficha']:
                    cod_ficha = corpo.supcor_codigo_ficha or False

                self.parent.add_item_in_tableitens(corpo.supcor_item, sequencia,
                                                   corpo.supcor_nome_item,
                                                   corpo.supcor_quantidade,
                                                   corpo.supcor_unidade,
                                                   corpo.supcor_preco_unitario,
                                                   corpo.supcor_preco_total,
                                                   [corpo.supcor_desconto_item,
                                                    corpo.supcor_acrescimo_item],
                                                   corpo.supcor_venda_cancelamento,
                                                   insert=False,
                                                   cod_ficha=cod_ficha)

            message_dlg(self.parent, MSG_INFO, 'Cupom %06d recuperado com sucesso!.' % coo)
            self.parent._coupon_id = coo
            self.parent._state_coupon = coupon_opened
            self.parent._coupon_subtotal = self.parent.send_command("get_coupon_subtotal()")

        self.get_totalizadores()

        return True

    @Log
    def open_coupon(self):
        """
        Abre o cupom fiscal na ECF e cria a capa do cupom no banco de dados.
        :return: Retorna os dados do cupom criado no banco de dados.
        """
        if not self.parent.user_logged or not self.parent.user_logged.id:
            message_dlg(self.parent, MSG_ERROR, 'Operador não está logado!')
            return False
        self.get_totalizadores(True)
        self.parent.limpa_variaveis_abertura_cupom()
        if config['usagaveta']:
            self.parent.obj_fiscal_printer.get_status_gaveta()

        finalizadoras = CaixaCadFinalizadora.query.order_by('name').limit(1).all()
        if not finalizadoras:
            raise Exception('Não há finalizadoras cadastradas na base de dados.')

        if not self.parent.get_printer_status():
            raise Exception("Não foi possível abrir o cupom de venda.")

        if self.parent.cliente.name:
            self.identificar_cliente(endereco=config['imprimirendereco'],
                                     cidade=config['imprimircidade'],
                                     insc_estadual=config['imprimirinscricaoestadual'],
                                     enviar_para_ecf=True)
        open_coupon = self.parent.send_command("open()")
        if not open_coupon:
            return False

        # NF Legal
        uf = self.parent.loja.supcfc_estado or ''
        uf = uf.upper()
        if uf == 'DF':
            self.parent.get_tot_tax()
            Log.info('Nota Legal DF')

        data_ini = self.parent.obj_fiscal_printer.get_data_ecf()
        data_cupom = self.parent.send_command('get_data_mov()')
        # Todo: ao abrir o cupom, algumas ECFs retornam o coo. Nao executar 'get_coo()' quando
        # Todo: o memso for retornado.
        nro_cupom = self.parent.send_command('get_coo()')
        self.parent.ui.lblNumCaixa.setText(u'%03d <b>Cupom #: %06d' % (self.parent.caixa_id,
                                                                       nro_cupom))
        self.parent._coupon_id = nro_cupom

        # --------------------------------- Cria a capa do cupom --------------------------------- #
        vals_capa = dict(supcap_data_cupom=data_cupom,
                         supcap_loja=self.parent.loja.supcfc_loja,
                         supcap_caixa=self.parent.caixa_id,
                         supcap_nro_cupom=nro_cupom,
                         supcap_operador_id=self.parent.user_logged.id,
                         supcap_cliente=self.parent.cliente.supcli_cnpj_cpf,
                         supcap_data_inicio=data_ini,
                         supcap_data_fim=self.parent.obj_fiscal_printer.get_data_ecf(),
                         supcap_tipo='V',
                         supcap_valor_desc_acres=0,
                         supcap_total_cupom=0,
                         supcap_mf_adicional=False,
                         supcap_nro_proprietario=1,
                         supcap_tipo_comprovante='RV',
                         supcap_subtotal=0,
                         supcap_indicador_desconto='V',
                         supcap_indicador_acrescimo='V',
                         supcap_indicador_cancelamento='N',
                         supcap_cancelamento_acrescimo=0,
                         supcap_cancelado_posteriormente=False,
                         supcap_nome_cliente=self.parent.cliente.name,
                         supcap_cod_vendedor=self.parent.vendedor_codigo)
        if self.parent._IdDav:
            vals_capa['supcap_numero_pedido'] = self.parent._IdDav.supdva_nro_pedido

        res = self.parent.obj_mov_capa.create(vals_capa)
        # ------------------------------------------ // ------------------------------------------ #

        self.parent._coupon = self.parent.mov_capa = res

        if self.parent._IdPve:
            b_sess.begin()
            self.parent._IdPve.NroCupom = nro_cupom
            b_sess.commit()

        return res

    def verifica_restricao_item(self, codigo_barras, qtd_item):
        """
        Verifica se o item em questao possui algum tipo de restricao
        :param codigo_barras: Codigo de barras do item vendido
        :param qtd_item: quantidade vendida
        :return:
        """
        retricoes = self.parent.obj_restricao_item.get_by(supres_codigo_barra=codigo_barras)
        tipo_restricao = 0
        if not retricoes:
            return False

        if retricoes.supres_solicita_sup:
            tipo_restricao = retricao_item
        elif retricoes.supres_max and retricoes.supres_max < qtd_item:
            tipo_restricao = restricao_qtd

        if tipo_restricao and self.parent.user_logged.supope_operador_supervisor == 'O':
            msg = u"Informe ao Supervisor!\n" + RESTRICOES.get(tipo_restricao)
            dlgpwd = password_dlg(self.parent, msg_info=msg)
            if dlgpwd.retval:
                pass
            else:
                self.parent.ui.edtCodigo.clear()
                self.parent.ui.spinQtda.setValue(1)
                message_dlg(self.parent, MSG_INFO,
                            'Operador: %s.\n\nNão tem permissão para '
                            'executar esta operação' %
                            str(self.parent.user_logged.name))
                return True

        return False

    @Log
    def add_item(self, query_item, quantidade, vendedor_codigo=None, nome_vendedor=None,
                 discount_surcharge=[], serie='', cod_ficha=False, valor_ficha=0):
        """
        Adiciona um item ao cupom/ecf
        :param: cod_item: código do produto
        :rtype: id do item no cupom
        """
        if not quantidade:
            msg = '<center><h2><font color="red">A quantidade informada está incorreta.</font>' \
                  '</h2><br><center><h2><font color="white">Pressione <font color="yellow">[Sair]' \
                  '</font> para digitar a quantidade correta.</font></h2>'
            message_dlg(self.parent, MSG_NONE, msg)
            self.parent.ui.edtCodigo.setText(query_item.supite_codigo_item)
            self.parent._returnPressed_edtCodigo()
            QtGui.QApplication.processEvents()
            return self.parent.KEY_QTDA_Click()

        if self.verifica_restricao_item(query_item.supite_codigo_item, quantidade):
            return False

        self.parent._lock_keyboard(True)
        self.parent.ui.edtCodigo.clear()

        item_id = False
        str_serie = ', No. Serie: %s' % serie if serie else serie

        # Verifica se é kit
        itens_kit, quantidades, vr_unitario_no_kit, descontos_kit_item = CadKits(self.parent).\
                                                        get_itens_kit(query_item.supite_codigo_item)

        codigo_kit = False
        kits_vendidos = {}
        qtd_kit = quantidade
        desc_acres_rateado = []
        if itens_kit:
            codigo_kit = query_item.supite_codigo_item
            itens = itens_kit
            if discount_surcharge:
                desc_acres_rateado = CadKits(self.parent).rateia_desc_acres_kit(quantidades,
                                                 vr_unitario_no_kit, qtd_kit, discount_surcharge[1])
        else:
            itens = query_item

        if not isinstance(itens, list):
            itens = [itens]

        inc = 0
        for item in itens:
            desconto_item_kit = 0
            tributacao_id = item.supite_tributacao_id
            cad_tributacao = CadTributacao.get_tributacao_by_id(tributacao_id)
            if not cad_tributacao:
                msg = 'O Código: %d, não está cadastrado da tabela de Tributações' % tributacao_id
                message_dlg(self.parent, MSG_ERROR, msg)
                QtGui.QApplication.processEvents()
                return False

            cod_trib_ecf = str(cad_tributacao.suptri_codigo_trib_ecf)
            try:
                item_price = item.supite_preco if not self.parent.vlr_item_dav else self.parent.vlr_item_dav
                if itens_kit:
                    quantidade = qtd_kit * quantidades[inc] or Decimal(1)
                    item_price = vr_unitario_no_kit[inc] or Decimal(0)
                    desconto_item_kit = descontos_kit_item[inc] or Decimal(0)

                qtd_item = quantidade
                # este valor eh referente ao valor da ficha que pode ser diferente do valor do item
                # caso a ficha foi vendida a um valo X e depois o item mudou para Y
                if valor_ficha:
                    item_price = valor_ficha

                item_price = Decimal(item_price).quantize(TWOPLACES)


                desconto_acrescimo = (0, 0)

                item_amount = item_price * qtd_item

                if discount_surcharge:
                    state_item = discount_surcharge[0]
                    value = discount_surcharge[1]
                    # desconto por valor rateado para itens do kit
                    if desc_acres_rateado:
                        value = desc_acres_rateado[inc]

                    desconto_acrescimo = self.get_desconto_acrescimo(state_item, value, item_amount,
                                                                     item_price, qtd_item)
                acrescimo_kit_item = 0
                # quando o desconto for negativo, quer dizer que o kit custa mais caro que a soma
                # dos itens do kit, portando precisa dar um acrescimo.
                if desconto_item_kit < 0:
                    acrescimo_kit_item = desconto_item_kit * -1
                    desconto_item_kit = 0
                    desconto_acrescimo = (0, desconto_acrescimo[1])
                    self.parent._state_item = item_add_surchage
                    self.parent.ui.tableItens.horizontalHeaderItem(5).setText(u'Acrésc.')
                else:
                    self.parent.ui.tableItens.horizontalHeaderItem(5).setText('Desc.')


                desconto_acrescimo = (desconto_acrescimo[0] + desconto_item_kit * qtd_kit,
                                      desconto_acrescimo[1] + acrescimo_kit_item * qtd_kit)
                und = item.supite_unidade
                if not und or len(und) < 2:
                    und = 'UN'
                else:
                    und = und[:2]

                if self.parent._state_coupon == coupon_closed:
                    # --- Cria a capa do cupom --- #
                    open_coupon = self.open_coupon()
                    if open_coupon:
                        self.parent._state_coupon = coupon_opened
                    else:
                        self.parent._set_screen_state(in_menu_fiscal)
                        QtGui.QApplication.processEvents()
                        if self.parent._coupon_id:
                            msg = 'Erro ao gravar o cupom %06d no banco de dados.' \
                                  % self.parent._coupon_id
                        else:
                            msg = 'Erro ao abrir o cupom'

                        message_dlg(self.parent, MSG_ERROR, msg)
                        return False

                if self.parent.list_totalizadores and \
                            cod_trib_ecf not in self.parent.list_totalizadores and \
                            cod_trib_ecf.rjust(2, '0') not in self.parent.list_totalizadores:
                    msg = 'A tributação "%s", não está cadastrado na ECF!' % \
                          cod_trib_ecf.rjust(2,'0')
                    message_dlg(self.parent, MSG_ERROR, msg)
                    QtGui.QApplication.processEvents()
                    return False
                
                if self.parent.comanda_id:
                    for x in self.parent.vendedor_comanda_ids:
                        if query_item == x[1][0] and self.parent.last_item == x[0]:
                            vendedor_codigo = x[2]
                            nome_vendedor = x[3]
                    if type(vendedor_codigo) is not int:
                        vendedor_codigo = self.parent.vendedor_codigo = 0
                        nome_vendedor = self.parent._nome_vendedor = None

                desc = desconto_acrescimo[0] * -1 if desconto_acrescimo[0] < 0 else desconto_acrescimo[0]
                acre = desconto_acrescimo[1] * -1 if desconto_acrescimo[1] < 0 else desconto_acrescimo[1]
                descricao = ''
                if item.supite_cest:
                    descricao += item.supite_cest + '#'
                if item.supite_ncm:
                    descricao += item.supite_ncm + '#'

                descricao += item.name.rstrip() + str_serie
                ecf_subtotal = self.parent.send_command("get_coupon_subtotal()")
                item_preco_total = 0
                # conversao de tributacao para elgin
                if self.parent._printer.brand == 'elgin':
                    if cod_trib_ecf == 'F':
                        cod_trib_ecf = '-2'
                    elif cod_trib_ecf == 'I':
                        cod_trib_ecf = '-3'
                    elif cod_trib_ecf in ['O', 'N']:
                        cod_trib_ecf = '-4'

                # caso o subtotal depois da venda seja igual ao anterio, nao vendeu o item, e repete
                # o comando de venda
                while -0.001 < item_preco_total and item_preco_total < 0.001:
                    item_id = self.parent.send_command(
                        '''add_item(item_code=%r, item_description=%r, item_price=%r, taxcode=%r,
                           items_quantity=%r, unit=%s, discount=%r, surcharge=%r, unit_desc=%r)''' % (
                            item.supite_codigo_item, descricao,
                            item_price, cod_trib_ecf, qtd_item,
                            'UnitType.CUSTOM', desc, acre, und))

                    while True:
                        try:
                            total_cupom = self.parent.send_command("get_coupon_subtotal()")
                            break
                        except Exception as e:
                            Log.error(e)

                    item_preco_total = total_cupom - ecf_subtotal
                    # espera 0.2 segundo para tentar novamente
                    if -0.001 < item_preco_total and item_preco_total < 0.001:
                       time.sleep(0.2)

                # ----- Fim do item na ecf------

                # nao conseguiu pegar o id do item na ecf
                if item_id == 999:
                    Log.error('Nao coseguiu pegar o numero do item: %s na ECF. \n'
                             'Recuperando via banco...' % item.supite_codigo_item)
                    item_id = self.parent.last_item + 1

                if not item_id:
                    raise ItemAdditionError('Não foi possível vender o item!')

                self.parent.last_item = item_id
                item_name = item.name if config['descricao_resumida'] else item.supite_descricao_completa
                supcor_preco_total = item_preco_total - acre + desc
                aliq_ipbt_fed = Decimal(item.supite_aliquota_ibpt_fed or 0.0)
                aliq_ipbt_est = Decimal(item.supite_aliquota_ibpt_est or 0.0)
                aliq_ipbt_mun = Decimal(item.supite_aliquota_ibpt_mun or 0.0)
                vr_imposto_fed = Decimal(supcor_preco_total * aliq_ipbt_fed / 100).quantize(TWOPLACES)
                vr_imposto_est = Decimal(supcor_preco_total * aliq_ipbt_est / 100).quantize(TWOPLACES)
                vr_imposto_mun = Decimal(supcor_preco_total * aliq_ipbt_mun / 100).quantize(TWOPLACES)
                chave_ibpt = item.supite_ibpt_chave

                # ----------------------------- Cria o corpo do cupom ---------------------------- #
                vals_item = dict(supcor_sequencia=item_id,
                                 supcor_item=item.supite_codigo_item,
                                 supcor_quantidade=qtd_item,
                                 supcor_nro_serie_item=serie,
                                 supcor_preco_total=supcor_preco_total,
                                 supcor_desconto_item=desc,
                                 supcor_desconto_item_sem_rat=desc,
                                 supcor_acrescimo_item=acre,
                                 supcor_acrescimo_item_sem_rat=acre,
                                 supcor_venda_cancelamento='V',
                                 supcor_nome_item=item_name,
                                 supcor_unidade=und,
                                 supcor_preco_unitario=item_price.quantize(TWOPLACES),
                                 supcor_tributacao_ecf=str(cad_tributacao.suptri_codigo_trib_ecf),
                                 supcor_indicador_arr_trunc='T',
                                 supcor_indicador_ppt='T',
                                 supcor_ncm=item.supite_ncm,
                                 supcor_cest=item.supite_cest,
                                 supcor_capa_id=self.parent.mov_capa.id,
                                 supcor_cod_vendedor=vendedor_codigo,
                                 supcor_nome_vendedor=nome_vendedor,
                                 supcor_vr_imposto_fed= vr_imposto_fed,
                                 supcor_vr_imposto_est= vr_imposto_est,
                                 supcor_vr_imposto_mun= vr_imposto_mun,
                                 supcor_ibpt_chave= chave_ibpt,
                                 supcor_nro_serie=self.parent.mov_capa.supcap_nro_serie or None)
                if config['usar_ficha'] and cod_ficha:
                    vals_item['supcor_codigo_ficha'] = cod_ficha

                if codigo_kit:
                    if codigo_kit not in kits_vendidos:
                        kit_vendido = MovKitsVendidos(self.parent).create(
                                                            {'supmkv_codigo_kit': codigo_kit,
                                                             'supmkv_quantidade': qtd_kit})
                        kit_vendido_id = kit_vendido.id
                        kits_vendidos.update({codigo_kit: kit_vendido_id})
                    else:
                        kit_vendido_id = kits_vendidos.get(codigo_kit)

                    vals_item['supcor_kit_vendido_id'] = kit_vendido_id

                self.parent.mov_corpos.append(MovCorpos(self.parent).create(vals_item))
                # -------------------------------------- // -------------------------------------- #

                self.parent.vlr_item_dav = False
                self.parent.add_item_in_tableitens(item.supite_codigo_item, item_id,
                                                   item_name, qtd_item, und,
                                                   item_price.quantize(TWOPLACES), item_preco_total,
                                                   desconto_acrescimo, 'V', insert=True,
                                                   cod_ficha=cod_ficha)
                vals_capa = dict(supcap_subtotal=total_cupom,
                                 supcap_total_cupom=total_cupom)
                self.parent.obj_mov_capa.write(self.parent.mov_capa, vals_capa)
                self.parent.ui.lblInfoAcresTotalItens.setText(
                    u'%1.2f' % (Decimal(self.parent.ui.lblInfoAcresTotalItens.text()) + acre))
                self.parent.ui.lblInfoDescTotalItens.setText(
                    u'%1.2f' % (Decimal(self.parent.ui.lblInfoDescTotalItens.text()) + desc))
                self.parent.desconto_total_itens += desc

                inc += 1

                self.parent.infoText('%r item(s) adicionado(s) ao cupom!' % self.parent.last_item,
                                     False)
                if self.parent.teclado:
                    self.parent.teclado.envia_linha_display(("Ad. item %s " %item.name)[:40], 2)
                    self.parent.linhas_display == 2 and self.parent.teclado.go_to(26, 1)

                self.parent.obj_fiscal_printer.update_file_gt()

            except ItemAdditionError:
                Log.error('**************************** NAO ADICIONOU ****************************')
                self.parent.vlr_item_dav=False
                self.parent.ui.spinQtda.setValue(1)
                self.parent.tratar_except()
                self.parent.check_button_status()
                QtGui.QApplication.processEvents()
                return False
            except Exception:
                Log.error('**************************** NAO ADICIONOU ****************************')
                self.parent.vlr_item_dav=False
                self.parent.ui.spinQtda.setValue(1)
                self.parent.tratar_except(tipo_msg=MSG_NONE)
                self.parent.check_button_status()
                QtGui.QApplication.processEvents()
                return False

        self.parent._lock_keyboard(False)
        self.parent.check_button_status()
        return item_id

    def set_vendedor(self):
        '''
        Grava o vendedor nos itens que nao o possuem(ex: selecionou o vendedor ao final da venda)
        '''
        mov_corpos =  MovCorpos(self.parent).search({'supcor_capa_id': self.parent.mov_capa.id,
                                          'supcor_venda_cancelamento': 'V'},
                                            order_by='supcor_sequencia') or []
        if mov_corpos and not isinstance(mov_corpos, list):
            mov_corpos = [mov_corpos]

        for corpo in mov_corpos:
            if corpo.supcor_cod_vendedor:
                continue

            corpo.supcor_cod_vendedor = self.parent.vendedor_codigo
            corpo.supcor_nome_vendedor = self.parent._nome_vendedor

        self.parent.mov_capa.supcap_cod_vendedor = self.parent.vendedor_codigo

        return True
    
    @Log
    def aplica_desc_socio_torcedor(self):
        '''
          Aplica o desconto do Socio Torcedor no Tef e na impressora
        '''
        if not self.parent._tef_nsu_socio_torcedor: # nao ha desconto para socio torcedor
            return False

        now = datetime.now()
        coo = str(self.parent.mov_capa.supcap_nro_cupom)
        data_fiscal = now.strftime('%Y%m%d')
        horario = now.strftime('%H%M%S')
        nome_operador = str(self.parent.user_logged.name)
        nsu_socio = self.parent._tef_nsu_socio_torcedor
        lista_produtos = ''
        sequencia = 1
        dict_desconto = {}
        desconto_total = 0
        # na impressora -----------------------
        for sequencia_item in sorted(self.parent.dict_mov_corpos_socio.keys()):
            corpo = self.parent.dict_mov_corpos_socio.get(sequencia_item).get('corpo')
            desconto = self.parent.dict_mov_corpos_socio.get(sequencia_item).get('desconto')
            codigo_produto = corpo.supcor_item[1:]
            if not desconto: # itens que nao tiveram desconto
                continue

            item = corpo.supcor_sequencia
            qtd = corpo.supcor_quantidade
            self.parent.send_command("desc_acres_item('D', %s, %s)" % (item, desconto))
            corpo.supcor_desconto_item = desconto
            corpo.supcor_desconto_item_sem_rat = desconto
            desconto_total += desconto
            if codigo_produto not in dict_desconto:
                dict_desconto[codigo_produto] = {'desconto': desconto, 'qtd': qtd}
            else:
                dict_desconto[codigo_produto]['desconto'] +=  desconto
                dict_desconto[codigo_produto]['qtd'] += qtd

        for codigo_produto in dict_desconto:
            # desconto em centavos e sem pontuacao
            desconto = str(Decimal(dict_desconto.get(codigo_produto).get('desconto')*Decimal(100))
                           .quantize(0)).replace('.','')
            qtd = dict_desconto.get(codigo_produto).get('qtd')
            lista_produtos += codigo_produto + ':' + desconto + ':' + str(int(qtd)) + ';'

        lista_produtos = str(lista_produtos[:-1])

        # no tef ------------------
        if lista_produtos:
            self.parent.sig_tef.registra_desconto_socio_torcedor(coo,
                                                                 data_fiscal,
                                                                 horario,
                                                                 nome_operador,
                                                                 nsu_socio,
                                                                 lista_produtos,
                                                                 sequencia)
        self.parent.dict_mov_corpos_socio = {}
        self.parent.controle_cancelamento_tef = True
        return "Desconto Total: %s" % desconto_total

    @Log
    def get_desconto_socio_torcedor(self):
        '''
        Esta funcao aplica no final da venda desconto para o socio torcedor caso haja algum produto na promocao.
        '''
        self.parent._tef_nsu_socio_torcedor = ler_arq_socio_torcedor()
        if not (self.parent.sig_tef.usando_sitef and self.parent._tef_nsu_socio_torcedor):
            return False

        self.parent.lista_itens_desconto_socio = '' # este campo sera atribuido no arquivo iterativo.py
        conta_produto = 1
        self.parent.dict_mov_corpos_socio = {}
        dict_itens_vendidos = {}
        desconto_promocao = map(lambda x: '%s' % x.get('sequencia_item'),
                            self.parent.list_itens_desconto_promo)
        piso_sequencia = 0
        if self.parent._printer.brand == 'bematech':
            piso_sequencia = len(self.parent.mov_corpos) > 300 and len(self.parent.mov_corpos) - \
                                                                   300 or 0

        for corpo in self.parent.mov_corpos:
            if corpo.supcor_venda_cancelamento == 'C':
                continue

            codigo_item = corpo.supcor_item[1:]
            sequencia = str(corpo.supcor_sequencia) # sequencia em que o item foi vendido na ECF
            # pega apenas os ultimos 300 itens, limitacao da ecf
            if sequencia < piso_sequencia:
                continue
            # retira o desconto do socio torcedor nos item que estao em alguma promocao
            if sequencia in desconto_promocao:
                continue

            self.parent.dict_mov_corpos_socio[sequencia] = {'corpo':corpo}
            if not dict_itens_vendidos.get(codigo_item):
                dict_itens_vendidos[codigo_item] = {'qtd': int(corpo.supcor_quantidade),
                                                    'vr_unitario': str((Decimal(corpo.supcor_preco_unitario).
                                                                        quantize(TWOPLACES))).replace('.','')}
            else:
                dict_itens_vendidos[codigo_item]['qtd'] += int(corpo.supcor_quantidade)

        self.parent.str_lista_itens_socio = '' # lista que sera enviada para o TEF para consulta de itens
        for codigo_item in dict_itens_vendidos:
            valor_unitario = dict_itens_vendidos.get(codigo_item).get('vr_unitario')
            qtd = str(dict_itens_vendidos.get(codigo_item).get('qtd'))
            self.parent.str_lista_itens_socio += codigo_item + ':' + valor_unitario + ':' + qtd + ';'
            conta_produto +=1
            if conta_produto > 200: # a cada 200 produtos 'consulta_registra_produtos_ambev' deve ser chamado
                conta_produto = 1
                self.parent.sig_tef.consulta_registra_produtos_ambev()
                self.parent.str_lista_itens_socio = ''

        if self.parent.str_lista_itens_socio:
            self.parent.sig_tef.consulta_registra_produtos_ambev()

        total_desconto = Decimal(0).quantize(TWOPLACES)
        dados_desconto = {}
        for linha in self.parent.lista_itens_desconto_socio.split(';'):
            if not linha:
                continue
            dados_produto = linha.split(':')

            # Produto (EAN) da tabela
            codigo_produto = dados_produto[0]
            dados_desconto[codigo_produto] = {}

            # 1 se for desconto em valor inteiro, 2 se for desconto em porcentagem.
            dados_desconto[codigo_produto]['tipo_desconto'] = dados_produto[1]

            # Valor do desconto UNITÁRIO. Tanto nos casos de desconto com porcentagem e fixos, este campo virá
            # com duas casas decimais sem o ponto ou a vírgula.
            dados_desconto[codigo_produto]['desconto'] = Decimal(dados_produto[2]) / Decimal(100)

            # 0 se for um produto sem limite para a concessão de descontos, 1 se for limite por quantidade,
            # 2 se for limite por valor.
            dados_desconto[codigo_produto]['tipo_limite'] = dados_produto[3]

            # Quantidade ou valor de produtos com descontos aprovados. Caso seja um valor, este campo virá com duas
            # casas decimais sem o ponto ou a vírgula. Este campo virá nulo caso seja um produto com oferta ilimitada
            # para a concessão de descontos.
            dados_desconto[codigo_produto]['limite'] = dados_produto[4] or 0

        desconto_concedido = {}
        desconto_parcial = False
        for sequencia in sorted(self.parent.dict_mov_corpos_socio.keys()):
            corpo = self.parent.dict_mov_corpos_socio.get(sequencia).get('corpo')
            codigo_produto = corpo.supcor_item[1:]
            if not dados_desconto.get(codigo_produto): # item sem desconto do socio torcedor
                continue

            tipo_desconto = dados_desconto.get(codigo_produto).get('tipo_desconto')
            desconto = dados_desconto.get(codigo_produto).get('desconto')
            tipo_limite = dados_desconto.get(codigo_produto).get('tipo_limite')
            limite = Decimal(dados_desconto.get(codigo_produto).get('limite')).quantize(TREPLACES)
            qtd = corpo.supcor_quantidade
            qtd_vendida = desconto_concedido.get(codigo_produto).get('qtd') if desconto_concedido.get(codigo_produto) \
                                                                            else 0
            if tipo_limite == '1': # limita o desconto por quantidade
                if qtd_vendida + qtd > limite:
                    qtd = limite - qtd_vendida
                    desconto_parcial = True
                    if -0.0001 < qtd and qtd < 0.0001: # desconto maximo aplicado
                        continue

                elif qtd > limite:
                    qtd = limite
                    desconto_parcial = True

            vr_desconto = desconto * qtd
            if tipo_desconto == '2': # desconto percentual
                # o arredondamento deve ser para baixo para a porcentagem calculada nao exceda o desconto consultado
                vr_desconto = Decimal(desconto * qtd * corpo.supcor_preco_unitario / Decimal(100)).\
                    quantize(TWOPLACES, rounding=ROUND_DOWN)

            valor_vendida = desconto_concedido.get(codigo_produto).get('valor') if desconto_concedido.get(
                                                                   codigo_produto) else Decimal(0).quantize(TWOPLACES)
            if tipo_limite == '2':  # limita o desconto por valor
                limite = Decimal(limite).quantize(TWOPLACES)/100
                if valor_vendida + vr_desconto  > limite:
                    vr_desconto = limite - valor_vendida
                    desconto_parcial = True
                    if -0.001 < vr_desconto and vr_desconto < 0.001: # desconto maximo aplicado
                        continue

                elif vr_desconto > limite:
                    desconto_parcial = True
                    vr_desconto = limite

            if not desconto_concedido.get(codigo_produto):
                desconto_concedido[codigo_produto] = {'qtd': qtd, 'valor': vr_desconto}
            else:
                desconto_concedido[codigo_produto]['qtd'] += qtd
                desconto_concedido[codigo_produto]['valor'] += vr_desconto

            self.parent.dict_mov_corpos_socio[sequencia]['desconto'] = vr_desconto
            total_desconto += vr_desconto

        self.parent.desconto_socio_torcedor = total_desconto
        if desconto_parcial:
            message_dlg(self.parent, MSG_INFO, '<font size="10">Atenção!</font>' +
                        '\n\n<font size="10"> Apenas parte dos produtos adquiridos tiveram desconto!</font>')

        self.parent.desconto_socio_calculado = True
        return True

    @Log
    def aplica_desc_promocao(self):
        if not self.parent.list_itens_desconto_promo:
            return 'Sem desconto'

        itens_ja_registrados = {}
        desconto_total = 0
        for dados_desc in sorted(self.parent.list_itens_desconto_promo, key=lambda x: x.get('sequencia_item')):
            item = dados_desc.get('sequencia_item')
            desconto = dados_desc.get('desconto')
            corpo = filter(lambda x: x.supcor_sequencia == item, self.parent.mov_corpos)
            corpo[0].supcor_id_promo = int(dados_desc.get('id_promo'))
            corpo[0].supcor_quantidade_promo = dados_desc.get('quantidade')
            corpo[0].supcor_tipo_promo = dados_desc.get('tipo_promo')
            if not desconto or desconto == 0:
                # os itens que ja gravaram desconto no banco nao podem ser gravados novamente no banco
                if itens_ja_registrados.get(corpo[0].supcor_item):
                    continue

                corpo[0].supcor_desconto_item = Decimal(0)
                corpo[0].supcor_desconto_item_sem_rat = Decimal(0)
                continue

            try:
                self.parent.send_command("desc_acres_item('D', %s, %s)" % (item, desconto))
                corpo[0].supcor_desconto_item = desconto
                corpo[0].supcor_desconto_item_sem_rat = desconto
                itens_ja_registrados[corpo[0].supcor_item] = True
                desconto_total += desconto
            except Exception as err:
                msg = 'Erro ao aplicar desconto no item %s!\n' % (corpo[0].supcor_nome_item)
                msg += str(err) # Todo: Verificar como proceder caso perca a conexao com a impressora
                Log.error(msg)

        self.parent.list_itens_desconto_promo = []
        return "Desconto Total: %s" % desconto_total

    def aplica_desc_funcionario(self):
        self.parent.obj_cad_funcionarios.aplica_desc_funcionario()

    @Log
    def aplica_desc_itens(self):
        '''
        Aplica desconto nos item ao final da venda para as Promocoes e Para o Socio Torcedor
        '''
        if self.parent._coupon_totalize: # cupom ja totalizado, nao precisa calcular o desconto
            return 'Cupom totalizado'

        # os descontos nao sao acumulativos
        self.aplica_desc_socio_torcedor()
        self.aplica_desc_promocao()
        self.aplica_desc_funcionario()

        return True

    @Log
    def add_payment(self, value, checa_tef=True, pfinalizadora=None, parcelas=0):
        if not value and not self.parent.ficha:
            self.parent._CONTROL_FOCUS.clear()
            return message_dlg(self.parent, MSG_ERROR, 'Informe o valor para pagamento!')

        if self.parent.vr_doacao and self.parent.vr_doacao > value:
            return message_dlg(self.parent, MSG_ERROR,
                               u'O valor informado deve ser igual ao total da doação')

        elif self.parent.vr_doacao and self.parent.vr_doacao < value:
            self.parent.ui.lblVlrTroco.setText(u'%1.2f' % (value - self.parent.vr_doacao))

        if self.parent.vr_ficha and self.parent.vr_ficha < value:
            self.parent.ui.lblVlrTroco.setText((u'%1.2f' % (value - self.parent.vr_ficha)))

        nro_banco = agencia = nro_cheque = conta = troco = 0

        forma_pgto = self.parent._coupon_payment_method

        query_finalizadora = None

        if forma_pgto == pm_dinheiro:
            query_finalizadora = CaixaCadFinalizadora.query.filter("name ilike 'dinheiro'").all()
            if query_finalizadora:
                query_finalizadora = query_finalizadora[0]
            control_focus = self.parent.ui.spinPgtoDinheiro
            lbl_info = self.parent.ui.lblInfoDinheiro

        elif forma_pgto == pm_cheque:
            index = self.parent.ui.cbChTipoCheque.currentIndex()
            finalizadora_name = self.parent.ui.cbChTipoCheque.itemText(index)
            query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name)
            control_focus = self.parent.ui.spinChValor
            lbl_info = self.parent.ui.lblInfoCheque
            nro_banco = self.parent.ui.spinChNroBanco.value()
            agencia = self.parent.ui.spinChNroAgencia.value()
            nro_cheque = self.parent.ui.spinChNroCheque.value()
            conta = int(self.parent.ui.spinChNroCC.text())

        elif forma_pgto == pm_cartao:
            usa_tef = False
            index = self.parent.ui.cbTipoTef.currentIndex()

            if self.parent.ui.cbTipoTef.itemText(index) == u'T.E.F - Integrado ao Sistema':
                usa_tef = True

            if not pfinalizadora:
                index = self.parent.ui.cbChTipoCartao.currentIndex()
                finalizadora_name = self.parent.ui.cbChTipoCartao.itemText(index)
                query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name,
                                                                 supfin_usa_tef=usa_tef)
            elif type(pfinalizadora) is int:
                query_finalizadora = CaixaCadFinalizadora.get_by(id=pfinalizadora)

            else:
                query_finalizadora = pfinalizadora

            control_focus = self.parent.ui.spinCartaoValor
            lbl_info = self.parent.ui.lblInfoCartao

        elif forma_pgto == pm_ticket:
            usa_tef = False
            index = self.parent.ui.cbTipoTicket.currentIndex()

            if self.parent.ui.cbTipoTicket.itemText(index) == u'T.E.F':
                usa_tef = True

            if not pfinalizadora:
                index = self.parent.ui.cbFinTicket.currentIndex()
                finalizadora_name = self.parent.ui.cbFinTicket.itemText(index)
                query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name,
                                                                 supfin_usa_tef=usa_tef)
            elif type(pfinalizadora) is int:
                query_finalizadora = CaixaCadFinalizadora.get_by(id=pfinalizadora,
                                                                 supfin_usa_tef=usa_tef)
            else:
                query_finalizadora = pfinalizadora

            control_focus = self.parent.ui.spinValorTicket
            lbl_info = self.parent.ui.lblInfoTicket

        elif forma_pgto == pm_outros:
            index = self.parent.ui.cbOTipoPgto.currentIndex()
            finalizadora_name = self.parent.ui.cbOTipoPgto.itemText(index)
            query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name)
            control_focus = self.parent.ui.spinOValor
            lbl_info = self.parent.ui.lblInfoOutros

        if not query_finalizadora:
            message_dlg(self.parent, MSG_ERROR, 'A Forma de pagamento selecionada '
                                         'não está cadastrada na tabela de finalizadoras.')
            return False

        total_a_pagar = Decimal(self.parent.ui.lblVlrTotalR.text())


        pgto_na_ecf = '%.2d' % query_finalizadora.supfin_codigo_na_ecf
        query_finalizadora_na_ecf = CaixaCadFinalizadoraNaEcf.get_by(id=pgto_na_ecf)

        if self.parent.vr_doacao and not self.parent._coupon_totalize:
            self.parent._coupon_totalize = self.parent.obj_doacao.efetua_doacao(
                                              config['totalizador_doacao'],
                                              query_finalizadora_na_ecf.supfec_codigo_na_ecf, self.parent.vr_doacao)
            if not self.parent._coupon_totalize:
                message_dlg(self.parent, MSG_ERROR, 'Atenção!\n'
                                                       'Erro ao tentar finalizar a Doação.\n'
                                                       'Tente novamente')
                self.parent._set_screen_state(in_add_payment)
                return False

        if self.parent.ficha and not self.parent._coupon_totalize:
            if self.parent.vr_ficha > 0:
                self.parent._coupon_totalize = self.parent.obj_ficha.recebimento_ficha(
                                              config['tot_ficha'], self.parent.vr_ficha)
            else: # troca de fichar
                self.parent.obj_ficha.extorno_ficha(config['tot_devolucao_ficha'],
                                                        abs(self.parent.vr_ficha),
                                                    query_finalizadora_na_ecf.supfec_codigo_na_ecf,
                                                    query_finalizadora)

                if config['usagaveta'] and not self.parent._PveCancelamento:
                    self.parent.send_command("open_cash_drawer(50)")
                    self.parent._timer_end_sale.start(500)
                else:
                    self.parent._timer_end_sale.start(1500)

                self.parent._clear_globals(0)
                self.parent._set_screen_state(in_end_sale)
                self.parent.show_info_proccess(False)
                self.parent._payment_receipt_g = []
                self.parent._texto_comprovante = None
                self.parent._payment_receipt = []
                self.parent._libera_restricao = False
                self._libera_desconto = False
                self.parent.ui.stackedActions.setCurrentIndex(stk_page_payments)
                self.parent.ui.stackedPgto.setCurrentIndex(pm_end)
                return value

        sequence = CaixaMovFechamentos.query.filter_by(supfec_capa_id=self.parent.mov_capa.id).count()
        sequence += 1
        self.set_vendedor()
        avista_aprazo = query_finalizadora.supfin_avista_aprazo
        tipo_finalizadora = query_finalizadora.supfin_tipo_finalizadora
        usa_tef = query_finalizadora.supfin_usa_tef

        emite_relatorio_g = False
        if self.parent._payment_amount + value > total_a_pagar:
            permite_troco = query_finalizadora.supfin_permite_troco
            if not permite_troco:
                return message_dlg(self.parent, MSG_INFO, 'Finalizadora não permite troco')

            troco = self.parent._payment_amount + value - total_a_pagar
            troco_maximo = query_finalizadora.supfin_troco_maximo

            if troco > troco_maximo:
                control_focus.setValue(total_a_pagar - self.parent._payment_amount)
                return message_dlg(self.parent, MSG_INFO, 'Troco excede valor permitido: '
                                                   '%.2f > %.2f' % (troco, troco_maximo))
            # Contra-Vale
            elif query_finalizadora.supfin_aceita_contra_vale and checa_tef:
                texto = 'COO............: %11d\n' % self.parent._coupon_id
                texto += 'Valor da Compra: %11.2f\n' % total_a_pagar
                valor = self.parent._payment_amount + value - total_a_pagar
                texto += 'Contra-Vale....: %11.2f\n' % valor
                self.add_payment_receipt_g(query_finalizadora, valor, sequence, texto)
                emite_relatorio_g = True

        libera_cliente = False
        limite_cliente = 0

        if self.parent.cliente.name and forma_pgto in (pm_outros, pm_cheque):
            libera_cliente = False if self.parent.cliente.supcli_status == 'B' else True


            if avista_aprazo == 'P':
                if not usa_tef:
                    limite_compras = self.parent.cliente.supcli_limite_compras or 0
                    limite = limite_compras - self.parent.cliente_total_aberto
                if limite >= value:
                    limite_cliente = 1

        dlg_info = ''
        if avista_aprazo == 'V' or tipo_finalizadora == 'C':
            limite_cliente = 1
            libera_cliente = True

        if not libera_cliente:
            if self.parent.cliente.name and avista_aprazo == 'P':
                dlg_info += '|CLI.BLOQ|'
            if tipo_finalizadora == 'H' and avista_aprazo == 'P':
                dlg_info += '|CH.PRZ|'
                if self.parent.cliente.name:
                    dlg_info += 'CLI.BLOQ|'

        if limite_cliente <= 0 and self.parent.cliente.name and avista_aprazo == 'P':
            dlg_info += '|LIM.CR|' if not dlg_info else 'LIM.CR|'
            if tipo_finalizadora == 'H':
                dlg_info += 'CH.PRZ|'

        tempo_sup_intervencao = config['temposupintervencao']
        vender_offline = False

        if self.parent._time_offline >= tempo_sup_intervencao > 0:
            vender_offline = True

        if forma_pgto in [pm_cartao, pm_ticket]:
            libera_cliente = limite_cliente = True

        if avista_aprazo == 'P' and not tipo_finalizadora == 'C'\
                and self.parent.user_logged.supope_operador_supervisor == 'O':
            if vender_offline and config['autorizacaosup']:
                dlgpwd = password_dlg(self.parent, msg_info='Supervisor OFFLINE!')
                if dlgpwd.retval:
                    self.parent._libera_restricao = dlgpwd.user.id
                else:
                    if emite_relatorio_g:
                        self.remove_payment_receipt_g()
                    return message_dlg(self.parent, MSG_INFO,
                                       'Operador: %s.\n\nNão tem permissão para '
                                       'executar esta operação' %
                                       str(self.parent.user_logged.name))
            else:
                self.parent._libera_restricao = self.parent.user_logged.id

        if avista_aprazo == 'P' and tipo_finalizadora not in ('C', 'T'):
            Log.info('Cliente: %s' % str(self.parent.cliente.supcli_codigo_interno))
            Log.info('Valor do pagamento: %s' % str(value))
            Log.info('Limite compra: %s' % str(self.parent.cliente.supcli_limite_compras or 0))
            Log.info('Total em aberto: %s' % str(self.parent.cliente_total_aberto or 0))
            Log.info('Operador: %s' % str(self.parent.user_logged.name))

        if avista_aprazo == 'P' and not tipo_finalizadora == 'C' \
                                and (not libera_cliente or limite_cliente <= 0)\
                                and not self.parent._IdDav:

                if message_dlg(self.parent, MSG_QUESTION,
                               'Deseja liberar esta venda? \n\n%s' % dlg_info) == NO_ROLE:
                    if emite_relatorio_g:
                        self.remove_payment_receipt_g()

                    return False

                if self.parent.user_logged.supope_operador_supervisor == 'O' and \
                        not self.parent.user_logged.supope_libera_restricao:
                    dlgpwd = password_dlg(self.parent, msg_info=dlg_info)
                    if dlgpwd.retval:
                        self.parent._libera_restricao = dlgpwd.user.id
                    else:
                        if emite_relatorio_g:
                            self.remove_payment_receipt_g()
                        return message_dlg(self.parent, MSG_INFO,
                                           'Operador: %s.\n\nNão tem permissão para '
                                           'executar esta operação' %
                                           str(self.parent.user_logged.name))
                else:
                    self.parent._libera_restricao = self.parent.user_logged.id

        libera_limite_credito = libera_cheque_avista = libera_restricao_msg = False
        if self.parent._libera_restricao and not self.parent._IdDav:
            if 'LIM.CR' in dlg_info:
                libera_limite_credito = True
            if 'CH.VIS' in dlg_info:
                libera_cheque_avista = True
            if 'CLI.BLOQ' in dlg_info:
                libera_restricao_msg = True

        self.aplica_desc_itens() # desconto de promocoes

        lbl_info.setText(u'%1.2f' % (Decimal(lbl_info.text()) + value))

        # Enviar o pagamento para a ECF
        if not self.parent._coupon_totalize:
            try:
                self.parent._coupon_totalize = Decimal(self.parent.send_command('totalize()'))
            except Exception as err:
                Log.error(err)
                try:
                    status = self.parent.send_command('get_status()')
                    status.check_error()
                except Exception as err:
                    if err.message == 'Impressora sem papel!' or type(err) == OutofPaperError:
                        while True:
                            message_dlg(self.parent, MSG_INFO,
                                        'Atenção!\n\n Impressora sem Papel! Favor colocar '
                                        'uma nova bobina para continuar')
                            try:
                                status = self.parent.send_command('get_status()')
                                status.check_error()
                                break
                            except Exception as e:
                                Log.error('%s' % e)

                    elif emite_relatorio_g:
                        self.remove_payment_receipt_g()
                        self.cancela_transacao_tef()
                        return False

        if troco > 0:
            self.parent.ui.lblVlrTroco.setText(u'%1.2f' % troco)

        nome_finalizadora_na_ecf = query_finalizadora_na_ecf.name

        if query_finalizadora_na_ecf and nome_finalizadora_na_ecf:
            try:
                pgto_na_ecf = self.parent.send_command('configure_payment_methods(%r)'
                                                       % str(nome_finalizadora_na_ecf))
                if not pgto_na_ecf:
                    message_dlg(self.parent, MSG_INFO,
                                'A forma de pagamento %r não está cadastrada na ECF!\n'
                                % nome_finalizadora_na_ecf +
                                'Não há espaço para cadastrar mais uma forma de pagamento!'
                                )
                    if emite_relatorio_g:
                        self.remove_payment_receipt_g()

                    return False

                while True:
                    if self.parent.vr_doacao: # nao faz os teste a baixo para doacao
                        break

                    tentativas = 0
                    while tentativas < 1:
                        tentativas += 1
                        try:
                            state = self.parent.send_command("get_status()", qtd_tentativas=1)
                            break
                        except Exception as e:
                            Log.error(str(e))
                            state = False

                    if state:
                        if self.parent.vr_ficha:
                            self.parent.obj_ficha.pagamento_nao_fiscal(pgto_na_ecf, value)
                        else:
                            self.parent.send_command('add_payment(%r, %r, %r)' %
                                                     (pgto_na_ecf, value, ''))
                        break

                    if message_dlg(self.parent, MSG_QUESTION,
                                'Impressora não responde. Deseja tentar novamente?') == NO_ROLE:
                        self.cancelar_cupom(tipo_cancelamento='N')
                        self.parent.desbloqueia_importacao()
                        self.parent.show_info_proccess(False)
                        self.parent._payment_receipt_g = []
                        self.parent._texto_comprovante = None
                        self.parent._payment_receipt = []
                        self.parent._libera_restricao = False
                        self._libera_desconto = False
                        now = datetime.now()
                        data_fiscal = now.strftime('%Y%m%d')
                        horario = now.strftime('%H%M%S')
                        if self.parent.sig_tef.usando_sitef:
                            self.parent.sig_tef.finalizar_transacao(0,
                                                              self.parent.mov_capa.supcap_nro_cupom,
                                                               data_fiscal, horario)

                        return self.parent._set_screen_state(in_coupon)

            except Exception as err:
                Log.error(str(err))
                if emite_relatorio_g:
                    self.remove_payment_receipt_g()

                self.cancela_transacao_tef()
                return False
        else:
            message_dlg(self.parent, MSG_INFO,
                        'A forma de pagamento %r não está cadastrada na tabela '
                        'caixa_cad_finalizadoras_na_ecf' % query_finalizadora.name)
            if emite_relatorio_g:
                self.remove_payment_receipt_g()

            return False

        if usa_tef or avista_aprazo == 'P':
            self.add_payment_receipt_g(query_finalizadora, value, sequence,
                                       texto=self.parent._texto_comprovante,
                                       parcelas=parcelas)

        self.parent._payment_amount += value
        self.parent.ui.lblVlrPgto.setText(u'%1.2f' % self.parent._payment_amount)

        if self.parent._payment_amount >= total_a_pagar:
            troco = self.parent._payment_amount - total_a_pagar
            self.parent.ui.lblVlrTroco.setText(u'%1.2f' % troco)
            self.parent.ui.lblVlrReceber.setText(u'%1.2f' % 0.0)
            mov_corpos = MovCorpos(self.parent).search({'supcor_capa_id': self.parent.mov_capa.id,
                                             'supcor_venda_cancelamento': 'V'})

            if not isinstance(mov_corpos, list):
                mov_corpos = [mov_corpos]

            # ------------- Se tiver acréscimo/desconto na capa, efetua rateio entre os itens ------------ #
            itens_rateados = {}
            try:
                indicador_desc_acre = self.parent.mov_capa.supcap_alteracao
                valor_desc_acres = Decimal(self.parent.ui.lblInfoDescCapa.text())
                if indicador_desc_acre and valor_desc_acres:
                    while True:
                        try:
                            totalizadores_pos_cupom = self.parent.send_command('get_totalizadores()')
                            break
                        except Exception as err:
                            if err.message == 'Timeout' or err == TimeoutError:
                                if message_dlg(self.parent, MSG_QUESTION,
                                        'Impressora não responde. Deseja tentar novamente?') == NO_ROLE:

                                    self.cancelar_cupom('N')
                                    sitef = self.parent.sig_tef.usando_sitef
                                    message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nCupom cancelado!\n'
                                                'Transação TEF cancelada!\nReter o cupom!' if sitef else '')
                                    self.parent._set_screen_state(in_coupon)
                                    return False

                    totalizadores = []
                    if not self.totalizadores_ant_cupom:
                        self.get_totalizadores()

                    Log.info('Totalizadores antes da venda: %s' % str(self.totalizadores_ant_cupom))
                    Log.info('Totalizadores depois da venda: %s' % str(totalizadores_pos_cupom))
                    for i in range(len(self.totalizadores_ant_cupom) or len(totalizadores_pos_cupom)):
                        # caso o tipo de totalizador seja diferente continua Ex: ICMS e ISS
                        if totalizadores_pos_cupom[i][2] != self.totalizadores_ant_cupom[i][2]:
                            continue

                        tot = (totalizadores_pos_cupom[i][0],
                               totalizadores_pos_cupom[i][1]-self.totalizadores_ant_cupom[i][1],
                               totalizadores_pos_cupom[i][2])
                        totalizadores.append(tot)

                    itens_rateados = MovCorpos(self.parent).set_rateio_desc_acre_itens(totalizadores,
                                                                            indicador_desc_acre,
                                                                            mov_corpos)

            except Exception as e:
                Log.error(str(e))
            # ---------------------------------------- // ---------------------------------------- #


            total_imposto_fed = Decimal('0.0')
            total_imposto_est = Decimal('0.0')
            total_imposto_mun = Decimal('0.0')

            for corpo in mov_corpos:
                preco_total = corpo.supcor_preco_total
                desconto = corpo.supcor_desconto_item
                acrescimo = corpo.supcor_acrescimo_item
                total_item = Decimal(preco_total - desconto + acrescimo)
                item = CaixaCadItens.get_by(supite_codigo_item=corpo.supcor_item,
                                            supite_loja=self.parent.loja.supcfc_loja)
                aliq_ipbt_fed = Decimal(item.supite_aliquota_ibpt_fed or 0.0)
                aliq_ipbt_est = Decimal(item.supite_aliquota_ibpt_est or 0.0)
                aliq_ipbt_mun = Decimal(item.supite_aliquota_ibpt_mun or 0.0)
                total_imposto_fed += Decimal(total_item * aliq_ipbt_fed / 100).quantize(TWOPLACES)
                total_imposto_est += Decimal(total_item * aliq_ipbt_est / 100).quantize(TWOPLACES)
                total_imposto_mun += Decimal(total_item * aliq_ipbt_mun / 100).quantize(TWOPLACES)

                vals_corpo = {}
                if itens_rateados and itens_rateados.get(corpo.id):
                    if indicador_desc_acre == 'D':
                        vals_corpo['supcor_desconto_item'] = itens_rateados.get(corpo.id).get('supcor_desconto_item')
                    else:
                        vals_corpo['supcor_acrescimo_item'] = itens_rateados.get(corpo.id).get('supcor_acrescimo_item')
                    MovCorpos(self.parent).write(corpo, vals_corpo)

            while True:
                try:
                    time.sleep(0.1)
                    self.parent.obj_fiscal_printer.printer._port.setTimeout(0.3)
                    total_cupom = self.parent.send_command('get_coupon_subtotal()')
                    self.parent.obj_fiscal_printer.printer._port.setTimeout(1)
                    break
                except Exception as err:
                    if err.message == 'Timeout' or type(err) == TimeoutError:
                        if message_dlg(self.parent, MSG_QUESTION,
                                'Impressora não responde. Deseja tentar novamente?') == NO_ROLE:

                            self.parent.obj_fiscal_printer.printer._port.setTimeout(1)
                            self.cancelar_cupom('N')
                            sitef = self.parent.sig_tef.usando_sitef
                            message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nCupom cancelado!\n'
                               'Transação TEF cancelada!\nReter o cupom!' if sitef else '')
                            self.parent._set_screen_state(in_coupon)
                            return False


            # esta parte ja foi feita para doacao e ficha
            if not self.parent.vr_doacao and not self.parent.vr_ficha:
                data_fim = self.parent.obj_fiscal_printer.get_data_ecf()
                self.parent.obj_mov_capa.write(self.parent.mov_capa,
                                {'supcap_vr_imposto_fed': total_imposto_fed,
                                 'supcap_vr_imposto_est': total_imposto_est,
                                 'supcap_vr_imposto_mun': total_imposto_mun,
                                 'supcap_cliente': self.parent.cliente.supcli_cnpj_cpf,
                                 'supcap_nome_cliente': self.parent.cliente.name,
                                 'supcap_operador_id': self.parent.user_logged.id,
                                 'supcap_cod_vendedor': self.parent.vendedor_codigo,
                                 'supcap_total_cupom': total_cupom,
                                 'supcap_subtotal': total_cupom,
                                 'supcap_data_fim': data_fim,})

            # Imprime rodapé do cupom
            if not self.print_promotional_message():
                return False

            self.parent.ui.stackedActions.setCurrentIndex(stk_page_payments)
            self.parent.ui.stackedPgto.setCurrentIndex(pm_end)
            self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.stackedPgto)
            self.parent._state_coupon = coupon_finish

            self.parent.ui.lblNumCaixa.setText(u'%03d <b>Aberto' % self.parent.caixa_id)

            if self.parent._IdPve:
                try:
                    Log.info('Gravando PVE #: %s' % self.parent._IdPve.id)
                    b_sess.begin()
                    self.parent._IdPve.suppva_data_cupom = self.parent._date_coupon
                    self.parent._IdPve.suppva_nro_cupom = self.parent._coupon_id
                    self.parent._IdPve.suppva_loja = self.parent.loja.supcfc_loja
                    self.parent._IdPve.suppva_caixa = self.parent.caixa_id
                    self.parent._IdPve.suppva_restricao = 2
                    b_sess.commit()
                except Exception as err:
                    Log.error(str(err))

            elif self.parent.comanda_id:
                try:
                    b_sess.begin()
                    for comanda in self.parent._IdsComandas:
                        comanda.supcda_nro_cupom = self.parent.mov_capa.supcap_nro_cupom
                        comanda.supcda_loja = self.parent.loja.supcfc_loja
                        comanda.supcda_caixa = self.parent.caixa_id
                        comanda.supcda_restricao = 2
                    b_sess.commit()
                except Exception as err:
                    Log.error(str(err))

            finaliza_venda = True

        else:
            vr_receber = total_a_pagar - self.parent._payment_amount
            self.parent.ui.lblVlrReceber.setText(u'%1.2f' % vr_receber)
            self.parent._set_screen_state(in_add_payment)
            finaliza_venda = False

        # ------------------------------ Grava fechamentos do cupom ------------------------------ #
        self.parent.mov_fechamentos = MovFechamentos(self.parent).create(
            dict(supfec_sequencia=sequence,
                 supfec_finalizadora=query_finalizadora.supfin_codigo,
                 supfec_tipo_finalizadora=query_finalizadora.supfin_tipo_finalizadora,
                 supfec_valor=value,
                 supfec_troco=troco,
                 supfec_nro_banco=nro_banco,
                 supfec_agencia=agencia,
                 supfec_nro_cheque=nro_cheque,
                 supfec_conta=conta,
                 supfec_cnpj=self.parent.cnpj_cpf_cheque,
                 supfec_nome_finalizadora=query_finalizadora.name,
                 supfec_capa_id=self.parent.mov_capa.id))

        # -------------------------------- Grava meios_pagamento --------------------------------- #
        MovMeiosPagamentos().gravar_meios_pagamento(self.parent.loja.supcfc_loja,
                                                    self.parent.caixa_id,
                                                    self.parent._date_coupon,
                                                    self.parent.mov_capa.supcap_tipo,
                                                    self.parent.mov_capa.supcap_tipo_comprovante,
                                                    value, query_finalizadora.supfin_codigo,
                                                    query_finalizadora.name)
        if self.parent._IdPve:
            try:
                b_sess.begin()
                SupervisorPveFechamento(
                    suppvf_capa_id=self.parent._IdPve.id, suppvf_sequencia=sequence,
                    suppvf_finalizadora=query_finalizadora.supfin_codigo, suppvf_valor=value,
                    suppvf_cheque_banco=nro_banco,
                    suppvf_cheque_agencia=agencia, suppvf_cheque_numero=nro_cheque,
                    suppvf_cheque_conta=conta,
                    suppvf_cheque_juros=0, suppvf_cheque_cnpj=self.parent.cnpj_cpf_cheque
                )
                b_sess.commit()
                self._IdPve = False
            except Exception as err:
                Log.error(str(err))

        if self.parent.cliente.name and forma_pgto in (pm_outros, pm_cheque):
            if avista_aprazo == 'P':
                if not usa_tef:
                    # a atualizacao do total em aberto somente deve ser feita apos a venda ser finalizada
                    self.parent.cliente_total_aberto += value
                    CadClientes(self.parent).atualiza_total_em_aberto(self.parent.loja.supcfc_loja,
                                                           self.parent.cliente,
                                                           self.parent.cliente_total_aberto)

        pg_com_cartao = False
        usou_tef = False
        erro = False
        if finaliza_venda:
            fin_tef = False
            finaliza_tef = False
            for p in self.parent._payment_receipt_g:
                texto_tef = ''
                mov_fechamentos = MovFechamentos(self.parent).search(
                    {'supfec_capa_id': self.parent.mov_capa.id,
                     'supfec_sequencia': p[3]})
                if mov_fechamentos:
                    for x in p[4]:
                        MovVencimentos().create(
                            dict(supfve_sequencia=p[3],
                                 supfve_vencimento=x[1],
                                 supfve_valor_parcela=x[2],
                                 supfve_sequencia_vencimento=x[0],
                                 supfve_fechamentos_id=mov_fechamentos.id))

                try:
                    if 'S' in [p[5], p[6]]:
                        self.parent.infoText('Imprimindo Gerencial')
                        tipo = 'P'
                        if p[6] == 'S':
                            tipo = 'O'
                        GerencialReport(report_id=config['rg_relatorio_gerencial'],
                                        text=p[1],
                                        loja_id=self.parent.loja.supcfc_loja,
                                        caixa_id=self.parent.caixa_id,
                                        operador_id=self.parent.user_logged.id,
                                        nro_cupom_vinculado=self.parent.mov_capa.supcap_nro_cupom,
                                        tipo=tipo,
                                        total=p[2],
                                        parent=self.parent)
                    if p[6] == 'T':
                        self.parent.infoText('Imprimindo Vinculado')
                        texto_tef += p[1]
                        query_finalizadora_na_ecf = CaixaCadFinalizadoraNaEcf.get_by(
                            id=p[0])

                        if query_finalizadora_na_ecf:
                            pgto_na_ecf = self.parent.send_command(
                                'configure_payment_methods(%r)' % str(query_finalizadora_na_ecf.name))

                            fin_tef = (1, self.parent.mov_capa.supcap_nro_cupom,
                                       pgto_na_ecf, p[2],)

                        else:
                            message_dlg(self, MSG_INFO,
                                        'A forma de pagamento %r não está cadastrada na tabela '
                                        % str(query_finalizadora.name) +
                                        'caixa_cad_finalizadoras_na_ecf')
                        erro = False
                        comprovante = False
                        report_id = text = None
                        fechei_receipt = False
                        while True:
                            try:
                                # Imprime recibo do pagamento
                                msg = 'Imprimindo comprovante do pagamento...'
                                self.parent.teclado and self.parent.teclado.envia_display(msg)
                                self.parent.show_info_proccess(True, msg)
                                self.parent.send_command("payment_receipt_open(%r, %r, %r, %r)" % fin_tef)
                                comprovante = True
                                self.parent.send_command("payment_receipt_print(%r)" % texto_tef)
                                self.parent.send_command("payment_receipt_close()")
                                fechei_receipt = True
                                self.parent.show_info_proccess(False)
                                break

                            except Exception as err:
                                Log.error(str(err))
                                # case de erro ao imprimir o comprovante de credido/debito, ira imprimir
                                # um relatorio gerencial
                                report_id = config['rg_relatorio_gerencial']
                                text = texto_tef
                                if err.message == 'Timeout' or type(err) == TimeoutError:
                                    if message_dlg(self.parent, MSG_QUESTION,
                                            'Impressora não responde. Deseja tentar novamente?') == NO_ROLE:
                                        erro = True
                                        self.parent.desbloqueia_importacao()
                                        self.parent.show_info_proccess(False)
                                        self.parent._payment_receipt_g = []
                                        self.parent._texto_comprovante = None
                                        self.parent._payment_receipt = []
                                        self.parent._libera_restricao = False
                                        self._libera_desconto = False
                                        now = datetime.now()
                                        data_fiscal = now.strftime('%Y%m%d')
                                        horario = now.strftime('%H%M%S')
                                        if self.parent.sig_tef.usando_sitef:
                                            pg_com_cartao = True
                                            self.parent.sig_tef.finalizar_transacao(0, self.parent.mov_capa.supcap_nro_cupom,
                                                                               data_fiscal, horario)

                                        if comprovante:
                                            try:
                                                self.parent.send_command("payment_receipt_close()")
                                                fechei_receipt = True
                                            except Exception as e:
                                                Log.error("Erro: %s" % str(e))

                                        message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nTransação TEF cancelada!\nReter o cupom!')
                                        break

                                    if not comprovante: # caso nao tenha aberto o coprovante, tenta de novo
                                        continue

                                    try:
                                        self.parent.send_command("payment_receipt_close()")
                                        fechei_receipt = True
                                    except:
                                        pass

                                    self.parent.show_info_proccess(False)
                                    break

                                elif err.message == 'Impressora sem papel!' or type(err) == OutofPaperError:
                                    while True:
                                        message_dlg(self.parent, MSG_INFO,
                                                    'Atenção!\n\n Impressora sem Papel! Favor colocar '
                                                    'uma nova bobina para continuar')
                                        try:
                                            status = self.parent.send_command('get_status()')
                                            status.check_error()
                                            break
                                        except Exception as e:
                                            Log.error('%s' % e)

                                    if not comprovante:  # caso nao tenha aberto o coprovante, tenta de novo
                                        continue

                                    try:
                                        self.parent.send_command("payment_receipt_close()")
                                        fechei_receipt = True
                                    except:
                                        pass

                                    self.parent.show_info_proccess(False)
                                    break


                                # no caso de nem abrir o cupom por motivo de erro na cadastro da
                                # forma de pagamento 'comprovante' = True, 'fechei_receipt' = True
                                comprovante = True
                                fechei_receipt = True
                                message_dlg(self.parent, MSG_INFO,
                                            'A finalizadora %s na ECF não está cadastrada para '
                                            'transação tef. Favor contactar ao seu supervisor'
                                            % pgto_na_ecf)
                                self.parent.show_info_proccess(False)
                                break

                        if comprovante: # se abriu o comprovante de pagamento grava o mesmo no banco
                            while True:
                                try:
                                    if not fechei_receipt:
                                        self.parent.send_command("payment_receipt_close()")
                                        fechei_receipt = True
                                except:
                                    pass

                                try:
                                    # imprime um RG com os mesmos dados do Comprovande de
                                    # Credito/Debito que ocorreu erro caso contrario apenas grava
                                    # o cupom
                                    GerencialReport(report_id=report_id,
                                                text=text,
                                                loja_id=self.parent.loja.supcfc_loja,
                                                caixa_id=self.parent.caixa_id,
                                                operador_id=self.parent.user_logged.id,
                                                nro_cupom_vinculado=self.parent.mov_capa.supcap_nro_cupom,
                                                tipo='T',
                                                tipo_c='CC',
                                                total=p[2],
                                                parent=self.parent)
                                    break
                                except Exception as e:
                                    Log.error(str(err))
                                    if err.message == 'Timeout' or err == TimeoutError:
                                        if message_dlg(self.parent, MSG_QUESTION,
                                                'Impressora não responde. Deseja tentar novamente?') == NO_ROLE:
                                            erro = True
                                            self.parent.desbloqueia_importacao()
                                            self.parent.show_info_proccess(False)
                                            self.parent._payment_receipt_g = []
                                            self.parent._texto_comprovante = None
                                            self.parent._payment_receipt = []
                                            self.parent._libera_restricao = False
                                            self._libera_desconto = False
                                            now = datetime.now()
                                            data_fiscal = now.strftime('%Y%m%d')
                                            horario = now.strftime('%H%M%S')
                                            if self.parent.sig_tef.usando_sitef:
                                                pg_com_cartao = True
                                                self.parent.sig_tef.finalizar_transacao(0, self.parent.mov_capa.supcap_nro_cupom,
                                                                                   data_fiscal, horario)

                                            message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nTransação TEF cancelada!\nReter o cupom!')
                                            break

                        if erro: # erro na impressao dos TEF
                            self.parent.show_info_proccess(False)
                            break

                        finaliza_tef = True
                        self.parent.show_info_proccess(False)

                except Exception as err:
                    Log.error(str(err))
                    self.parent.show_info_proccess(False)

            if (finaliza_tef or not pg_com_cartao) and self.parent.sig_tef.usando_sitef:
                self.parent.show_info_proccess(True, 'Finalizando Transações SigTef')
                now = datetime.now()
                data_fiscal = now.strftime('%Y%m%d')
                horario = now.strftime('%H%M%S')
                self.parent.sig_tef.finalizar_transacao(1, self.parent.mov_capa.supcap_nro_cupom,
                                                       data_fiscal, horario)
                self.parent.show_info_proccess(False, '')

            if self.parent.codigo_estacionamento and not erro:
                while True:
                    try:
                        res = self.parent.obj_nepos.envia_codigo_barras()
                        coo = self.parent.mov_capa.supcap_nro_cupom
                        self.parent.KEY_RETURN_Click()
                        msg = 'COO vinculado: %s' % coo + '\n' + \
                              'Valor da compra: %s' % str(self.parent.mov_capa.supcap_total_cupom).\
                                                                         replace('.',',') + '\n' + \
                              res[1].split('\x00')[0] + '\n' + \
                              'No. Ticket:' + self.parent.codigo_estacionamento
                        if res[0]:
                            GerencialReport(report_id=config['rg_relatorio_gerencial'],
                                            text=msg,
                                            loja_id=self.parent.loja.supcfc_loja,
                                            caixa_id=self.parent.caixa_id,
                                            operador_id=self.parent.user_logged.id,
                                            tipo='G',
                                            parent=self.parent)
                        msg = res[1]
                        if not res[0]:
                            msg = '%s Cartao: %s' % (msg, self.parent.codigo_estacionamento)

                        message_dlg(self.parent, MSG_INFO, msg)
                        break
                    except Exception as e:
                        Log.error(str(e))
                        msg = 'Erro ao validar cartão: \n\n' + str(e) + '\n\n Deseja tentar novamente?'
                        if message_dlg(self.parent, MSG_QUESTION, msg) == NO_ROLE:
                            break

                self.parent.codigo_estacionamento = ''

            vals_aut = dict(supaut_cliente=self.parent.cliente.supcli_cnpj_cpf,
                            supaut_capa_id=self.parent.mov_capa.id,
                            supaut_liberou_limite_credito=False,
                            supaut_liberou_cheque_a_vista=False,
                            supaut_liberou_restricao_mensagem=False,
                            supaut_liberou_desconto=False)

            if self.parent._libera_desconto:
                if self.parent._libera_restricao == self.parent._libera_desconto:
                    vals_aut['supaut_autorizador_id'] = self.parent._libera_desconto
                    vals_aut['supaut_liberou_desconto'] = True
                else:
                    vals_aut_desc = vals_aut.copy()
                    vals_aut_desc['supaut_autorizador_id'] = self.parent._libera_desconto
                    vals_aut_desc['supaut_liberou_desconto'] = True
                    MovAutorizacoes().create(vals_aut_desc)

            if self.parent._libera_restricao:
                vals_aut['supaut_autorizador_id'] = self.parent._libera_restricao
                vals_aut['supaut_liberou_limite_credito'] = libera_limite_credito
                vals_aut['supaut_liberou_cheque_a_vista'] = libera_cheque_avista
                vals_aut['supaut_liberou_restricao_mensagem'] = libera_restricao_msg

            if vals_aut.get('supaut_autorizador_id'):
                MovAutorizacoes().create(vals_aut)

            QtGui.QApplication.processEvents()
            # ------------------- Grava cupom servidor ------------------- #

            cupserv_id = MovCupomServidor().search({'supcse_capa_id': self.parent.mov_capa.id})
            if not cupserv_id:  # caso não exista
                MovCupomServidor().create({'supcse_capa_id': self.parent.mov_capa.id})

            CadSorteio(self.parent).verifica_sorteio()
            if config['usagaveta'] and not self.parent._PveCancelamento:
                self.parent.send_command("open_cash_drawer(50)")
                self.parent._timer_end_sale.start(500)
            else:
                self.parent._timer_end_sale.start(1500)

            if self.parent._IdMesa:
                self.parent._IdMesa.DataHoraFechamento = datetime.now()
                self.parent._IdMesa.Status = 'F'

            if self.parent._IdDav:
                try:
                    Log.info('Salvando DAV')

                    b_sess.begin()
                    self.parent._IdDav.supdva_data_cupom = self.parent._date_coupon
                    self.parent._IdDav.supdva_loja = self.parent.loja.supcfc_loja
                    self.parent._IdDav.supdva_caixa = self.parent.caixa_id
                    self.parent._IdDav.supdva_operador = self.parent._operador_id
                    self.parent._IdDav.supdva_nro_cupom = self.parent.mov_capa.supcap_nro_cupom
                    self.parent._IdDav.supdva_restricao = 2
                    b_sess.commit()
                    if config['aplicativo_dav'] == 'SiG-Gestao':
                        o_dav = DavGestao(config.get('ip_gestao'), 5019)
                        o_dav.fatura_dav(self.parent._IdDav.id)
                        b_sess.begin()
                        self.parent._IdDav.supdva_restricao = 4
                        b_sess.commit()
                    self.parent._IdDav = False
                except Exception as err:
                    Log.error(str(err))

            if self.parent.vr_ficha:
                self.parent.obj_ficha.cria_cupom_fichas(self.parent.vr_ficha)

            if self.parent.vr_doacao or self.parent.vr_ficha:
                self.parent._clear_globals(0)
                self.parent._set_screen_state(in_end_sale)

            self.parent.show_info_proccess(False)
            self.parent._payment_receipt_g = []
            self.parent._texto_comprovante = None
            self.parent._payment_receipt = []
            self.parent._libera_restricao = False
            self._libera_desconto = False

        return value

    @Log
    def remove_payment_receipt_g(self):
        if self.parent._payment_receipt_g:
            self.parent._payment_receipt_g.pop() # retira o ultimo elemento da lista

        return True

    @Log
    def cancela_transacao_tef(self):
        if self.parent.sig_tef.usando_sitef:
            self.parent.show_info_proccess(True, 'TEF cancelado')
            now = datetime.now()
            data_fiscal = now.strftime('%Y%m%d')
            horario = now.strftime('%H%M%S')
            self.parent.sig_tef.finalizar_transacao(0,
                                                    self.parent.mov_capa.supcap_nro_cupom,
                                                    data_fiscal, horario)
            time.sleep(2)
            self.parent.show_info_proccess(False)
            return True

        return False

    @Log
    def add_payment_receipt_g(self, method, value, sequence, texto=None, parcelas=0):
        if parcelas:
            qtd_parcelas = parcelas
        else:
            if method.supfin_outros_qtd_parcelas > 0:
                qtd_parcelas = method.supfin_outros_qtd_parcelas
            else:
                qtd_parcelas = 1

        vlr_parcela = Decimal(value / qtd_parcelas).quantize(TWOPLACES)
        total_calculado = Decimal(vlr_parcela * qtd_parcelas).quantize(TWOPLACES)
        dif = Decimal(value - total_calculado).quantize(TWOPLACES)
        print_ = 'N'
        cc = 'N'  # contra-vale
        if method.supfin_imp_promissoria:
            print_ = 'S'
        elif method.supfin_aceita_contra_vale:
            cc = 'S'
        if method.supfin_usa_tef:
            print_ = cc = 'T'

        data = datetime.today().date()
        outros_intervalo_parcelas = method.supfin_outros_intervalo_parcelas or 0
        if method.supfin_tipo_finalizadora == 'H' and method.supfin_avista_aprazo == 'P':
            data = datetime.strptime(self.parent.ui.dateChVcto.text(), '%d/%m/%Y')
        elif method.supfin_outros_prazo_primeira:
            supfin_outros_prazo_primeira = method.supfin_outros_prazo_primeira
            data += timedelta(days=supfin_outros_prazo_primeira)

        elif method.supfin_outros_mensal_vencimento:
            supfin_outros_mensal_vencimento = method.supfin_outros_mensal_vencimento
            outros_mensal_fecha_mes = method.supfin_outros_mensal_fecha_mes
            qtd_parcelas = 1
            qtd_dias_no_mes = calendar.monthrange(data.year, data.month)[1]
            data = data + timedelta(days=qtd_dias_no_mes)  # soma 1 mes na data
            if data.day <= int(outros_mensal_fecha_mes or 0):
                qtd_dias_no_mes = calendar.monthrange(data.year, data.month)[1]
                if supfin_outros_mensal_vencimento < qtd_dias_no_mes:
                    data = data.replace(day=int(supfin_outros_mensal_vencimento))
                else:
                    data = data.replace(day=int(qtd_dias_no_mes))
            else:
                qtd_dias_no_mes = calendar.monthrange(data.year, data.month)[1]
                data = data + timedelta(days=qtd_dias_no_mes)   # soma 1 mes na data
                qtd_dias_no_mes = calendar.monthrange(data.year, data.month)[1]
                if supfin_outros_mensal_vencimento < qtd_dias_no_mes:
                    data = data.replace(day=int(supfin_outros_mensal_vencimento))
                else:
                    data = data.replace(day=int(qtd_dias_no_mes))

        data_valor_vencimentos = []
        primeira_parcela = vlr_parcela + dif # se houver diferenca, joga na primeira parcela
        if not texto:
            text = 'Reconheco a exatidao desta NOTA PROMISSORIA que pagarei a %s, CNPJ: %s. ' \
                   'Ou a sua ordem nos vencimentos indicados.\n\n' % (self.parent.loja.supcfc_cliente,
                                                                      self.parent.loja.supcfc_cnpj)
            text += 'Valor por Extenso: %s \n' % str(
                NumeroPorExtenso(value).extenso_unidade).upper()
            text += 'Nro. Cupom: %.06d \n \n' % self.parent._coupon_id
            text += 'Parcela Vencimento Valor \n'
            inc = 0
            for x in range(qtd_parcelas):
                if not inc: # primeira parcela
                    vlr_parcela_s = primeira_parcela
                else: # demais parcelas
                    data += timedelta(days=outros_intervalo_parcelas)
                    vlr_parcela_s = vlr_parcela

                data_valor_vencimentos.append((x + 1, data, vlr_parcela_s))
                text += '%7s %s R$ %.2f\n' % ((str(x + 1) + '/' + str(qtd_parcelas)),
                                              data.strftime("%d/%m/%Y"),
                                              vlr_parcela_s,)

                inc += 1
            if self.parent.cliente.name:
                text += ' \n \n \nAss. Emitente: --------------------------------- \n'
                text += '%s\n%s\n%s - %s \n%s %s\n' % (
                    self.parent.cliente.name, self.parent.cliente.supcli_cnpj_cpf,
                    self.parent.cliente.supcli_endereco,
                    self.parent.cliente.supcli_endereco_numero, self.parent.cliente.supcli_cep,
                    self.parent.cliente.supcli_cidade)
                text += '================================================ \n'
        else:
            text = texto
            inc = 0
            for x in range(qtd_parcelas):
                if inc > 0: # demais parcelas
                    data += timedelta(days=outros_intervalo_parcelas)
                    data_valor_vencimentos.append((x + 1, data, vlr_parcela))
                else: # primeira parcela
                    data_valor_vencimentos.append((x + 1, data, primeira_parcela))

                inc += 1

        self.parent._texto_comprovante = None

        text = remove_accents(text, cod='ISO-8859-1')
        self.parent._payment_receipt_g.append(('%.2d' % method.supfin_codigo_na_ecf, str(text),
                                               value, sequence, data_valor_vencimentos, print_, cc))
        return self.parent._payment_receipt_g

    @Log
    def identificar_cliente(self, endereco=False, cidade=False, insc_estadual=False,
                            enviar_para_ecf=False):
        try:
            str_end = []
            cnpj_cpf = self.parent.cliente.supcli_cnpj_cpf
            cliente_name = self.parent.cliente.name

            if endereco and self.parent.cliente.supcli_endereco:
                str_end.append('%s' % remove_accents(
                                                 self.parent.cliente.supcli_endereco).rstrip()[:42])

                if self.parent.cliente.supcli_endereco_numero:
                    str_end.append(', %s' % self.parent.cliente.supcli_endereco_numero)

            if cidade and self.parent.cliente.supcli_cidade:
                str_end.append(', %s' % self.parent.cliente.supcli_cidade.rstrip())

                if self.parent.cliente.supcli_estado:
                    str_end.append('-%s' % self.parent.cliente.supcli_estado)

            if insc_estadual and self.parent.cliente.supcli_insc_estadual:
                str_end.append(' - IE: %s' % self.parent.cliente.supcli_insc_estadual)

            res = ''.join(str_end)
            if enviar_para_ecf:
                self.parent.send_command("identify_customer(%r, %r, %r)" % (cliente_name[:30],
                                                                            res[:42],
                                                                            cnpj_cpf))
                res = self.parent.cliente_identificado = True
        except Exception as err:
            Log.error(str(err))
            res = None

        return res

    @Log
    def get_desconto_acrescimo(self, state_item, value, preco_total, preco_unit, quantidade):
        desconto_acrescimo = (0, 0)
        if value > 0:
            if state_item == item_add_discount:
                desconto = value
                preco_total -= desconto
                desconto_acrescimo = desconto * -1
                desconto_acrescimo = (desconto_acrescimo, 0)
                self.parent.ui.tableItens.horizontalHeaderItem(5).setText('Desc.')

                if desconto >= (preco_unit * quantidade):
                    self.parent.ui.lblItems.clear()
                    self.parent.ui.lblVlrItem.setText(u'0,00')
                    self.parent._lock_keyboard(False)
                    self.parent.check_button_status()
                    raise ValueError('Não é permitido desconto maior que valor do item.')

            elif state_item == item_add_surchage:
                acrescimo = value
                preco_total += acrescimo
                desconto_acrescimo = acrescimo
                desconto_acrescimo = (0, desconto_acrescimo)
                self.parent.ui.tableItens.horizontalHeaderItem(5).setText(u'Acrésc.')

                if acrescimo >= (preco_unit * quantidade):
                    self.parent.ui.lblItems.clear()
                    self.parent.ui.lblVlrItem.setText(u'0,00')
                    self.parent._lock_keyboard(False)
                    self.parent.check_button_status()
                    raise ValueError('Não é permitido acrescimo maior que valor do item.')

        return desconto_acrescimo

    @Log
    def cancelar_cupom(self, tipo_cancelamento='C'):
        coo = self.parent.send_command("get_coo()")
        data_inicio = self.parent.obj_fiscal_printer.get_data_ecf()
        data_cupom = datetime.date(data_inicio)
        query_capa = self.parent.obj_mov_capa.search({'supcap_loja': config['loja'],
                                       'supcap_caixa': config['caixa'],
                                       'supcap_nro_cupom': coo,
                                       'supcap_data_cupom': data_cupom})
        id_cupom = False
        self.parent.socio_torcedor = False
        # Finaliza a transacao do TEF como cancelada
        if self.parent.sig_tef.usando_sitef or self.parent._tef_nsu_socio_torcedor:
            nsu = self.parent.obj_cor_nsu.query.all()
            if nsu :
                nsu = nsu[0]
                nroCupom = nsu.supnsu_nro_cupom_venda
                horaTransacao = nsu.supnsu_tef_hora_transacao
                dataTransacao = nsu.supnsu_tef_data_transacao
                self.parent.sig_tef.finalizar_transacao(0, nroCupom, dataTransacao, horaTransacao)
            else:
                ''
            self.parent.sig_tef.usando_sitef = False
            self.parent._tef_nsu_socio_torcedor = False


        try:
            if not query_capa:  # se nao existe o cupom no banco de dados
                self._cancel()
                return coo

            cupom_aberto = self.parent.send_command("get_status().open")
            if query_capa.supcap_tipo not in ['C', 'V'] \
                    and not cupom_aberto:
                self.parent.show_info_proccess(False)
                message_dlg(self.parent, MSG_ERROR,
                            'Não foi possivel cancelar o cupom %06d\n\n'
                            'Talvez você esteja tentando cancelar um Cupom não fiscal.' % coo)
                return False

            # Caso a sangria e o ressuprimento precisarem ser anulados
            if cupom_aberto and query_capa.supcap_tipo in ['A', 'U']:
                tipo_cancelamento = "N"

            self.parent.show_info_proccess(True, msg='Cancelando Cupom No. %d' % coo)
            QtGui.QApplication.processEvents()
            self._cancel()
            self.parent.emitindo_dav = False
            while True:
                id_cupom = self.parent.send_command("get_coo()")
                if id_cupom and (id_cupom > coo or tipo_cancelamento == 'N'):
                    break
        except CancelItemError:
            self.parent.show_info_proccess(False)
            message_dlg(self.parent, MSG_ERROR,
                        'Não foi possivel cancelar o cupom %06d\n\n'
                        'Talvez você esteja tentando cancelar um Cupom de Cancelamento.' % coo)
            return False
        except Exception as err:
            new_coo = self.parent.send_command("get_coo()")
            # se os dois COOs sao iguais, nao emitiu o de cancelamento caso contrario deu erro, mas
            # conseguiu imprimir o cupom de cancelamento.
            if new_coo == coo:
                self.parent.show_info_proccess(False)
                Log.error(str(err))
                return False

        Log.info('Inicinando transações com banco de dados para cupom %s de cancelamento:' % id_cupom)

        capa_id = query_capa.id

        try:
            if tipo_cancelamento == 'N':
                self.parent.obj_mov_capa.write(query_capa, {'supcap_nro_cupom': id_cupom,
                                             'supcap_tipo': tipo_cancelamento,
                                             'supcap_tipo_comprovante': 'NC',
                                             'supcap_cancelado_posteriormente': 'F',
                                             'supcap_data_fim': self.parent.obj_fiscal_printer.get_data_ecf()})

                query_corpos = MovCorpos(self.parent).search({'supcor_capa_id': capa_id,
                                                   'supcor_venda_cancelamento': 'V'})

                if not isinstance(query_corpos, list):
                    query_corpos = [query_corpos]

                for corpo in query_corpos:
                    MovCorpos(self.parent).write(corpo, {'supcor_venda_cancelamento': 'C'})

                MovCupomServidor().create({'supcse_capa_id': capa_id})

            else:
                self.parent.obj_mov_capa.write(query_capa, {'supcap_cancelado_posteriormente': True})
                dict_capa = query_capa.to_dict()

                del dict_capa['id']

                dict_capa['supcap_nro_cupom'] = id_cupom
                dict_capa['supcap_tipo'] = tipo_cancelamento
                dict_capa['supcap_tipo_comprovante'] = 'NC'
                dict_capa['supcap_cancelado_posteriormente'] = 'F'
                dict_capa['supcap_data_inicio'] = data_inicio
                dict_capa['supcap_data_fim'] = self.parent.obj_fiscal_printer.get_data_ecf()

                del query_capa

                cancelamento_capa_id = self.parent.obj_mov_capa.create(dict_capa).id

                query_corpos = MovCorpos(self.parent).search({'supcor_capa_id': capa_id,
                                                   'supcor_venda_cancelamento': 'V'})

                if not isinstance(query_corpos, list):
                    query_corpos = [query_corpos]

                for corpo in query_corpos:
                    dict_corpo = corpo.to_dict()
                    del dict_corpo['id']

                    dict_corpo['supcor_capa_id'] = int(cancelamento_capa_id)
                    dict_corpo['supcor_venda_cancelamento'] = 'C'

                    MovCorpos(self.parent).create(dict_corpo)

                query_fechamentos = MovFechamentos(self.parent).search({'supfec_capa_id': capa_id})

                if not isinstance(query_fechamentos, list):
                    query_fechamentos = [query_fechamentos]

                for fechamento in query_fechamentos:
                    dict_fechamento = fechamento.to_dict()
                    del dict_fechamento['id']
                    dict_fechamento['supfec_capa_id'] = int(cancelamento_capa_id)

                    fechamento_id = MovFechamentos(self.parent).create(dict_fechamento).id

                    query_vencimentos = MovVencimentos().search({'supfve_fechamentos_id': fechamento.id})

                    if query_vencimentos:

                        if not isinstance(query_vencimentos, list):
                            query_vencimentos = [query_vencimentos]

                        for vencimento in query_vencimentos:
                            dict_vencimento = vencimento.to_dict()
                            del dict_vencimento['id']
                            dict_vencimento['supfve_fechamentos_id'] = int(fechamento_id)
                            MovVencimentos().create(dict_vencimento)

                MovCupomServidor().create({'supcse_capa_id': cancelamento_capa_id})

                Log.info('Fim das transações com banco de dados para cupom %s de cancelamento:'
                         % id_cupom)

        except Exception as err:
            Log.error(str(err))
            self.parent.show_info_proccess(False)
            return False

        self.parent.show_info_proccess(False)
        return id_cupom

    def print_promotional_message(self):
        if self.parent.vr_doacao:
            return True

        if self.parent.vr_ficha:
            self.parent.obj_ficha.finaliza_ficha()
            return True

        if config['usar_ficha']:
            self.parent.obj_ficha.faturar_fichas()

        try:

            uf = self.parent.loja.supcfc_estado or ''
            uf = uf.upper()
            nl = '"NL"' if uf == 'DF' else ''

            cnpj_cpf = '%s' % self.parent.cliente.supcli_cnpj_cpf or ''

            promotional_message = 'MD-5: %s%s\n' % (
                self.parent.obj_fiscal_printer.get_ecf_autorizada(config['loja'], config['caixa'],
                                                         os.path.join(root_path, 'pdvsig.sig'),
                                                         True).md5, nl)

            cnpj = re.sub('[^0-9]', '', '%s' % (self.parent.loja.supcfc_cnpj or ''))
            data = datetime.now().strftime('%d%m%y')
            total = self.parent._coupon_amount * Decimal('1e2')

            if uf == 'MG':
                promotional_message += 'MINAS LEGAL: %s %s %s\n' % (cnpj, data, total)

            elif uf == 'RJ':
                ie = self.parent.send_command('get_read_ie()')

                promotional_message += 'CUPOM MANIA, CONCORRA A PRÊMIOS\n'
                promotional_message += 'ENVIE SMS P/ 6789: %s%s%06d%04d\n' \
                                       % (ie[:8], data, self.parent.mov_capa.id, config['loja'])

            elif uf == 'PB':
                promotional_message += u'PARAÍBA LEGAL - RECEITA CIDADÃ\n'
                cpf_cli = re.sub('[^0-9]', '', cnpj_cpf)
                cpf_cli = cpf_cli[4:] if self.parent.cliente.supcli_tipo_pessoa == 'F' else cpf_cli
                promotional_message += 'TORPEDO PREMIADO: %s %s %s %s\n' % (cnpj, data,
                                                                            total, cpf_cli)

            elif uf == 'DF':
                tot_tax = self.parent.get_tot_tax()
                tot_icms = tot_tax.get('icms') - self.parent._tot_icms
                tot_isqn = tot_tax.get('isqn') - self.parent._tot_isqn
                if config.get('nf_legal'):
                    promotional_message += "ESTABELECIMENTO INCLUIDO NO PROGRAMA DE\n"
                    promotional_message += "CONCESSAO DE CREDITOS - LEI no 4.159/08.\n"
                promotional_message += "NOTA LEGAL: ICMS = %.2f ISS = %.2f\n" \
                                       % (tot_icms, tot_isqn)
                self.parent._tot_isqn = self.parent._tot_icms = Decimal('0')

            vr_imposto_fed = Decimal(self.parent.mov_capa.supcap_vr_imposto_fed or 0)
            vr_imposto_est = Decimal(self.parent.mov_capa.supcap_vr_imposto_est or 0)
            vr_imposto_mun = Decimal(self.parent.mov_capa.supcap_vr_imposto_mun or 0)
            # chave = self.parent.mov_corpos.supcor_ibpt_chave or ''
            # chave = chave[:5]

            # porcentagem_imposto_fed = vr_imposto_fed / self.parent.mov_capa.supcap_total_cupom * 100
            # porcentagem_imposto_est = vr_imposto_est / self.parent.mov_capa.supcap_total_cupom * 100
            # porcentagem_imposto_mun = vr_imposto_mun / self.parent.mov_capa.supcap_total_cupom * 100

            # por questao de espaco, se nao houver imp. municipal, nao imprime o imposto
            if vr_imposto_mun:
                promotional_message += "Apr.TribR$%.2fFedR$%.2fEstR$%.2fMunFonte:IBPT\n"  % (
                                                          vr_imposto_fed,
                                                          vr_imposto_est,
                                                          vr_imposto_mun)
            else:
                promotional_message += "Apr.Trib R$%.2f Fed R$%.2f Est Fonte:IBPT\n"  % (
                                                          vr_imposto_fed,
                                                          vr_imposto_est)

            if self.parent._IdDav:
                promotional_message += 'DAV:%.10d\n' % self.parent._IdDav.supdva_nro_pedido
            elif self.parent._IdPve:
                promotional_message += 'PV:%.10d\n' % self.parent._IdPve.id

            elif self.parent._IdsComandas:
                qtd_comandas = len(self.parent._IdsComandas)
                if qtd_comandas > 1:
                    promotional_message += 'Comandas:'
                    for i in self.parent._IdsComandas:
                        promotional_message += '%.08d' % i.supcda_codigo_comanda
                        if self.parent._IdsComandas.index(i) < qtd_comandas - 1:
                            promotional_message += ', '
                    promotional_message += '\n'
                else:
                    promotional_message += 'Comanda:%.08d\n' % \
                                           self.parent._IdsComandas[0].supcda_codigo_comanda

            elif self.parent._IdMesa:
                if not self.parent._IdMesa.NroCupom:
                    promotional_message += 'Consumo da Mesa %d - SEM EMISSAO DE ' \
                                           'CONFERENCIA DE MESA\n' \
                                           % (self.parent._IdMesa.supmes_num_mesa)
                else:
                    promotional_message += 'ECF: %03d - Conferencia da Mesa - COO %06d\n' % \
                                           (self.parent.caixa_id, self.parent._IdMesa.NroCupom)

            if self.parent._nome_vendedor:
                operador_vendedor = 'OP: %s  Vend: %s' % (self.parent.user_logged.name,
                                                           self.parent._nome_vendedor)
                promotional_message += operador_vendedor[:47] + '\n'
            else:
                promotional_message += 'OP: %s\n' % self.parent.user_logged.name

            cliente_identificado = self.parent._printer.coupon_is_customer_identified()
            if not cliente_identificado and not self.parent.cliente_identificado and self.parent.cliente.name:
                if config['imprimircpfcodigointerno']:
                    cnpj_cpf = str(self.parent.cliente.supcli_codigo_interno)

                promotional_message += 'CLIENTE: %s\n' % self.parent.cliente.name.rstrip()
                promotional_message += 'CNPJ/CPF: %s\n' % cnpj_cpf

                str_end = self.identificar_cliente(endereco=config['imprimirendereco'],
                                                   cidade=config['imprimircidade'],
                                                   insc_estadual=config['imprimirinscricaoestadual'],
                                                   enviar_para_ecf=False)
                if str_end:
                    promotional_message += 'END.: %s\n' % str(str_end)

                msg_clientes_imp = ''
                msg_clientes = CaixaCadMensagensClientes.query.filter("supclm_cnpj_cpf='%s'" % cnpj_cpf)
                for msg in msg_clientes:
                    if msg.supclm_tipo in ['V', 'I', 'A']:
                        msg_clientes_imp += '* %s\n' % msg.supclm_mensagem
                if msg_clientes_imp:
                    promotional_message += '%s\n' % msg_clientes_imp

            try:
                if self.parent._message_in_coupon:
                    promotional_message += '%s\n' % self.parent._message_in_coupon.replace(',.', '\n')
                mensagens = ''
                cfg_mensagens = CaixaCfgMensagens.get_by()
                if cfg_mensagens:
                    if cfg_mensagens.supcfm_mensagem_1:
                        mensagens += '%s\n' % cfg_mensagens.supcfm_mensagem_1
                    if cfg_mensagens.supcfm_mensagem_2:
                        mensagens += '%s\n' % cfg_mensagens.supcfm_mensagem_2
                    promotional_message += '%s' % mensagens
            except Exception as err:
                Log.error('Erro ao imprimir mensagem no cupom: %s \n %s' % (mensagens, str(err)))

            QtGui.QApplication.processEvents()
        except Exception as err:
            Log.error(str(err))

        # -------------------- Implementação para corrigir erro na daruma -------------------- #
        # Ao tentar enviar o comando de fechamento para o cupom e a ecf estiver sem papel o
        # sistema gera excessão. Para corrigir esse problema o sistema fica em loop até que a
        # impressora retorne que o comando foi executado com sucesso.

        while True:
            try:
                aux = []
                # mais de 8 linhas esta dando erro na mensagem
                # a maioria das ECFs aceitam no maximo 8 linhas, independente da quantidade total
                # de caracteres
                for linha in promotional_message.split('\n'):
                    if len(linha) > 48:
                        aux.append(linha[:48])
                        if len(linha) > 96:
                            aux.append(linha[48:96])
                            aux.append(linha[96:])
                        else:
                            aux.append(linha[48:])

                    else:
                        aux.append(linha)

                promotional_message = '\n'.join(aux[:8])[:320]
                self.parent.send_command("close(promotional_message=%r)" %
                                         (u'%s'.encode('cp850') %
                                          promotional_message),
                                         test_status_time_out=False)
                break
            except CouponNotOpenError:
                self.parent._printer._has_been_totalized = False
                break
            except Exception as err:
                if err.message == 'Timeout' or type(err) == TimeoutError:
                    if message_dlg(self.parent, MSG_QUESTION,
                            'Impressora não responde. Deseja tentar novamente?') == NO_ROLE:

                        self.cancelar_cupom('N')
                        sitef = self.parent.sig_tef.usando_sitef
                        message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nCupom cancelado!\n'
                               'Transação TEF cancelada!\nReter o cupom!' if sitef else '')
                        self.parent._set_screen_state(in_coupon)
                        return False
                elif type(err) == OutofPaperError:
                    while type(err) == OutofPaperError:
                        try:
                            status = self.parent.send_command('get_status()')
                            status.check_error()
                            if status.open:
                                self.parent.send_command("close(promotional_message=%r)" %
                                                         (u'%s'.encode('cp850') %
                                                          promotional_message),
                                                         test_status_time_out=False)
                            break

                        except OutofPaperError as e:
                            err = e
                            message_dlg(self.parent, MSG_INFO,
                                        '%s\nNão consegui fechar o cupom. '
                                        'Troque a bobina de papel!' % str(err))
                        except CouponOpenError:
                            # para a bematec cupom aberto retorna erro na funcao check_erro
                            self.parent.send_command("close(promotional_message=%r)" %
                                                     (u'%s'.encode('cp850') %
                                                      promotional_message),
                                                     test_status_time_out=False)
                            break

                        except Exception as e:
                            pass

                    return True
                elif message_dlg(self.parent, MSG_QUESTION,
                               '%s\n\nNão consegui fechar o cupom. '
                               'Deseja tentar novamente? ' % str(err)) == NO_ROLE:
                    self.cancelar_cupom('N')
                    self.parent._set_screen_state(in_menu_fiscal)

        # ------------------------------------------------------------------------------------ #

        return True