#!/usr/bin/python
# -*- encoding: utf-8 -*-

__autor__ = 'Dener Rodrigues'

from sqlalchemy.exc import SQLAlchemyError
from sigpdv.control.mov_capa import MovCapa
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.tools.log import traced as Log
from decimal import Decimal
from datetime import datetime
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from sigpdv.ui.messagebox import MSG_ERROR, NO_ROLE, MSG_QUESTION, message_dlg, MSG_INFO
from stoqdrivers.exceptions import CodigoInvalido, OutofPaperError, TimeoutError


class GerencialReport(object):
    @Log
    def __init__(self, report_id=None, text=None, loja_id=None, caixa_id=None, operador_id=None,
                 nro_cupom_vinculado=None, tipo='G', tipo_c='RG', total=Decimal(0), parent=None):
        """
        Se informar report_id e text, o relatório será impresso,
        caso contrário, só grava a capa.

        :param report_id: codigo do relatório no ecf
        :param text: texto à imprimir
        :param loja_id: id da loja
        :param caixa_id: id do caixa
        :param operador_id: id do operador
        :param nro_cupom_vinculado: numero do cupom referente à este relatório
        :param tipo: tipo do cupom
        :param tipo_c: tipo do comprovante
        :param total: total do cupom
        """

        self.parent = parent
        self.printer = self.parent.obj_fiscal_printer

        gerar_capa = True

        if report_id is not None and text:
            if not self.imprimir(report_id, text):
                gerar_capa = False

        if gerar_capa:
            self.gerar_capa_rg(loja_id=loja_id, caixa_id=caixa_id, operador_id=operador_id,
                               nro_cupom_vinculado=nro_cupom_vinculado, tipo=tipo,
                               tipo_c=tipo_c, total=total)

    @Log
    def imprimir_ficha(self, report_id, text):
        tam = len(text)
        cont = 1
        abriu_cupom = False
        for i in text:
            while True:
                try:
                    if not abriu_cupom:
                        self.printer.send_command("gerencial_report_open(%r)" % report_id,
                                                  qtd_tentativas=1)
                        abriu_cupom = True

                    if cont == tam:  # retira o ultimo corta_papel
                        self.printer.send_command("gerencial_report_print(%r)" %
                                                  str('\n'.join(i.split('\n')[:-1])),
                                                  qtd_tentativas=1)
                    else:
                        self.printer.send_command("gerencial_report_print(%r)" % str(i),
                                                  qtd_tentativas=1)

                    cont += 1
                    break

                except CodigoInvalido as err:
                    try:
                        # tenta fechar o relatorio gentencial caso haja erro
                        self.printer.send_command("gerencial_report_close(%r)" %
                                                  self.parent.user_logged.name)
                    except:
                        pass
                    message_dlg(self, MSG_ERROR, str(err),
                                'O código(%d) do RG cadastrado no PDV-Config não '
                                'existe na ECF.' % report_id)

                    return False

                except Exception as e:
                    Log.error('%s' % e)
                    if 'Relatório gerencial não programado' in str(e):
                        try:
                            # tenta fechar o relatorio gentencial caso haja erro
                            self.printer.send_command("gerencial_report_close(%r)" %
                                                      self.parent.user_logged.name)
                        except:
                            pass

                        message_dlg(self.parent, MSG_INFO, str(e))
                        return False

                    self.parent.show_info_proccess(False)
                    message_dlg(self.parent, MSG_INFO,
                                'Atenção!\nErro ao imprimir ficha, verifique se acabou o '
                                'papel ou contacte o supervisor!\nTroque a bobina.')

                    self.parent.show_info_proccess(True, 'Imprimindo Fichas...')
                    try:
                        self.printer.send_command("get_corta_papel()", qtd_tentativas=1)
                    except:
                        pass

    @Log
    def imprimir(self, report_id, text):
        res = True
        try:
            # para ficha eh enviado uma lista com os textos para controle de eventuais falhas
            if isinstance(text, list):
                self.imprimir_ficha(report_id, text)
            else:
                self.printer.send_command("gerencial_report_open(%r)" % report_id, qtd_tentativas=1)
                self.printer.printer._port.setTimeout(0.5)
                self.printer.send_command("gerencial_report_print(%r)" % text, qtd_tentativas=1)
                self.printer.printer._port.setTimeout(1)

            self.printer.send_command("gerencial_report_close(%r)" % self.parent.user_logged.name,
                                      qtd_tentativas=1)
        except CodigoInvalido as err:
            message_dlg(self, MSG_ERROR, str(err),
                        'O código(%d) do RG cadastrado no PDV-Config não '
                        'existe na ECF.' % report_id)
            res = False

        except Exception as e:
            Log.error(str(e))
            if 'Relatório gerencial não programado' in str(e):
                message_dlg(self.parent, MSG_INFO, str(e))
                return False

            res = False
            self.printer.printer._port.setTimeout(0.5)
            while True:
                if type(e) == TimeoutError:
                    msg = 'Atenção\n\nFalha de comunicação com a impressora!\nTente novamente'
                else:
                    msg = 'Atenção!\n\n Impressora sem Papel! Favor colocar ' \
                          'uma nova bobina para continuar'

                message_dlg(self.parent, MSG_INFO, msg)
                try:
                    # tenta fechar o relatorio gentencial caso haja erro
                    self.printer.send_command("gerencial_report_close(%r)" %
                                              self.parent.user_logged.name)
                except:
                    pass

                try:
                    status = self.parent.send_command('get_status()')
                    status.check_error()
                    self.printer.send_command("gerencial_report_open(%r)" % report_id,
                                              qtd_tentativas=1)
                    self.printer.send_command("gerencial_report_print(%r)" % text,
                                              qtd_tentativas=1)
                    self.printer.send_command("gerencial_report_close(%r)" % self.parent.user_logged.name,
                                              qtd_tentativas=1)
                    self.printer.printer._port.setTimeout(1)
                    res = True
                    break
                except Exception as e:
                    Log.error('%s' % e)

            self.printer.printer._port.setTimeout(1)

        return res

    @Log
    def gerar_capa_rg(self, loja_id, caixa_id, operador_id, nro_cupom_vinculado=None, tipo='G',
                      tipo_c='RG', total=Decimal(0)):

        try:
            data_ini = self.printer.get_data_ecf() or datetime.now()
            data_cupom = datetime.date(data_ini)
            coo = self.printer.send_command("get_coo()")
            res = query_capa = MovCapa(self.parent).create(dict(
                supcap_data_cupom=data_cupom,
                supcap_loja=loja_id,
                supcap_caixa=caixa_id,
                supcap_nro_cupom=coo,
                supcap_operador_id=operador_id,
                supcap_data_inicio=data_ini,
                supcap_data_fim=data_ini,
                supcap_total_cupom=total,
                supcap_tipo=tipo,
                supcap_nro_proprietario=loja_id,
                supcap_tipo_comprovante=tipo_c,
                supcap_nro_cupom_vinculado=nro_cupom_vinculado if tipo == 'T' else None))

            # Cupom Servidor
            MovCupomServidor().create({'supcse_capa_id': query_capa.id})

            # Data Fim
            data_fim = self.printer.get_data_ecf() or datetime.now()
            MovCapa().write(query_capa, {'supcap_data_fim': data_fim})
        except SQLAlchemyError as err:
            Log.error(str(err))
            res = False

        return res